"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Defines the structure of protobuf messages for working with Live Streams
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _EnumLens:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumLensEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumLens.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    LENS_WIDE: _EnumLens.ValueType
    LENS_LINEAR: _EnumLens.ValueType
    LENS_SUPERVIEW: _EnumLens.ValueType

class EnumLens(_EnumLens, metaclass=_EnumLensEnumTypeWrapper): ...

LENS_WIDE: EnumLens.ValueType
LENS_LINEAR: EnumLens.ValueType
LENS_SUPERVIEW: EnumLens.ValueType
global___EnumLens = EnumLens

class _EnumLiveStreamError:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumLiveStreamErrorEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumLiveStreamError.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    LIVE_STREAM_ERROR_NONE: _EnumLiveStreamError.ValueType
    "No error (success)"
    LIVE_STREAM_ERROR_NETWORK: _EnumLiveStreamError.ValueType
    "General network error during the stream"
    LIVE_STREAM_ERROR_CREATESTREAM: _EnumLiveStreamError.ValueType
    "Startup error: bad URL or valid with live stream server"
    LIVE_STREAM_ERROR_OUTOFMEMORY: _EnumLiveStreamError.ValueType
    "Not enough memory on camera to complete task"
    LIVE_STREAM_ERROR_INPUTSTREAM: _EnumLiveStreamError.ValueType
    "Failed to get stream from low level camera system"
    LIVE_STREAM_ERROR_INTERNET: _EnumLiveStreamError.ValueType
    "No internet access detected on startup of streamer"
    LIVE_STREAM_ERROR_OSNETWORK: _EnumLiveStreamError.ValueType
    "Error occured in linux networking stack. Usually means the server closed the connection"
    LIVE_STREAM_ERROR_SELECTEDNETWORKTIMEOUT: _EnumLiveStreamError.ValueType
    "Timed out attemping to connect to the wifi network when attemping live stream"
    LIVE_STREAM_ERROR_SSL_HANDSHAKE: _EnumLiveStreamError.ValueType
    "SSL handshake failed (commonly caused due to incorrect time / time zone)"
    LIVE_STREAM_ERROR_CAMERA_BLOCKED: _EnumLiveStreamError.ValueType
    "Low level camera system rejected attempt to start live stream"
    LIVE_STREAM_ERROR_UNKNOWN: _EnumLiveStreamError.ValueType
    "Unknown"
    LIVE_STREAM_ERROR_SD_CARD_FULL: _EnumLiveStreamError.ValueType
    "Can not perform livestream because sd card is full"
    LIVE_STREAM_ERROR_SD_CARD_REMOVED: _EnumLiveStreamError.ValueType
    "Livestream stopped because sd card was removed"

class EnumLiveStreamError(_EnumLiveStreamError, metaclass=_EnumLiveStreamErrorEnumTypeWrapper): ...

LIVE_STREAM_ERROR_NONE: EnumLiveStreamError.ValueType
"No error (success)"
LIVE_STREAM_ERROR_NETWORK: EnumLiveStreamError.ValueType
"General network error during the stream"
LIVE_STREAM_ERROR_CREATESTREAM: EnumLiveStreamError.ValueType
"Startup error: bad URL or valid with live stream server"
LIVE_STREAM_ERROR_OUTOFMEMORY: EnumLiveStreamError.ValueType
"Not enough memory on camera to complete task"
LIVE_STREAM_ERROR_INPUTSTREAM: EnumLiveStreamError.ValueType
"Failed to get stream from low level camera system"
LIVE_STREAM_ERROR_INTERNET: EnumLiveStreamError.ValueType
"No internet access detected on startup of streamer"
LIVE_STREAM_ERROR_OSNETWORK: EnumLiveStreamError.ValueType
"Error occured in linux networking stack. Usually means the server closed the connection"
LIVE_STREAM_ERROR_SELECTEDNETWORKTIMEOUT: EnumLiveStreamError.ValueType
"Timed out attemping to connect to the wifi network when attemping live stream"
LIVE_STREAM_ERROR_SSL_HANDSHAKE: EnumLiveStreamError.ValueType
"SSL handshake failed (commonly caused due to incorrect time / time zone)"
LIVE_STREAM_ERROR_CAMERA_BLOCKED: EnumLiveStreamError.ValueType
"Low level camera system rejected attempt to start live stream"
LIVE_STREAM_ERROR_UNKNOWN: EnumLiveStreamError.ValueType
"Unknown"
LIVE_STREAM_ERROR_SD_CARD_FULL: EnumLiveStreamError.ValueType
"Can not perform livestream because sd card is full"
LIVE_STREAM_ERROR_SD_CARD_REMOVED: EnumLiveStreamError.ValueType
"Livestream stopped because sd card was removed"
global___EnumLiveStreamError = EnumLiveStreamError

class _EnumLiveStreamStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumLiveStreamStatusEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumLiveStreamStatus.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    LIVE_STREAM_STATE_IDLE: _EnumLiveStreamStatus.ValueType
    "Initial status. Livestream has not yet been configured"
    LIVE_STREAM_STATE_CONFIG: _EnumLiveStreamStatus.ValueType
    "Livestream is being configured"
    LIVE_STREAM_STATE_READY: _EnumLiveStreamStatus.ValueType
    "Livestream has finished configuration and is ready to start streaming"
    LIVE_STREAM_STATE_STREAMING: _EnumLiveStreamStatus.ValueType
    "Livestream is actively streaming"
    LIVE_STREAM_STATE_COMPLETE_STAY_ON: _EnumLiveStreamStatus.ValueType
    "Live stream is exiting. No errors occured."
    LIVE_STREAM_STATE_FAILED_STAY_ON: _EnumLiveStreamStatus.ValueType
    "Live stream is exiting. An error occurred."
    LIVE_STREAM_STATE_RECONNECTING: _EnumLiveStreamStatus.ValueType
    "An error occurred during livestream and stream is attempting to reconnect."
    LIVE_STREAM_STATE_UNAVAILABLE: _EnumLiveStreamStatus.ValueType
    "Live stream setup is unavailable due to camera lens configuration"

class EnumLiveStreamStatus(_EnumLiveStreamStatus, metaclass=_EnumLiveStreamStatusEnumTypeWrapper): ...

LIVE_STREAM_STATE_IDLE: EnumLiveStreamStatus.ValueType
"Initial status. Livestream has not yet been configured"
LIVE_STREAM_STATE_CONFIG: EnumLiveStreamStatus.ValueType
"Livestream is being configured"
LIVE_STREAM_STATE_READY: EnumLiveStreamStatus.ValueType
"Livestream has finished configuration and is ready to start streaming"
LIVE_STREAM_STATE_STREAMING: EnumLiveStreamStatus.ValueType
"Livestream is actively streaming"
LIVE_STREAM_STATE_COMPLETE_STAY_ON: EnumLiveStreamStatus.ValueType
"Live stream is exiting. No errors occured."
LIVE_STREAM_STATE_FAILED_STAY_ON: EnumLiveStreamStatus.ValueType
"Live stream is exiting. An error occurred."
LIVE_STREAM_STATE_RECONNECTING: EnumLiveStreamStatus.ValueType
"An error occurred during livestream and stream is attempting to reconnect."
LIVE_STREAM_STATE_UNAVAILABLE: EnumLiveStreamStatus.ValueType
"Live stream setup is unavailable due to camera lens configuration"
global___EnumLiveStreamStatus = EnumLiveStreamStatus

class _EnumRegisterLiveStreamStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumRegisterLiveStreamStatusEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumRegisterLiveStreamStatus.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    REGISTER_LIVE_STREAM_STATUS_STATUS: _EnumRegisterLiveStreamStatus.ValueType
    REGISTER_LIVE_STREAM_STATUS_ERROR: _EnumRegisterLiveStreamStatus.ValueType
    REGISTER_LIVE_STREAM_STATUS_MODE: _EnumRegisterLiveStreamStatus.ValueType
    REGISTER_LIVE_STREAM_STATUS_BITRATE: _EnumRegisterLiveStreamStatus.ValueType

class EnumRegisterLiveStreamStatus(
    _EnumRegisterLiveStreamStatus, metaclass=_EnumRegisterLiveStreamStatusEnumTypeWrapper
): ...

REGISTER_LIVE_STREAM_STATUS_STATUS: EnumRegisterLiveStreamStatus.ValueType
REGISTER_LIVE_STREAM_STATUS_ERROR: EnumRegisterLiveStreamStatus.ValueType
REGISTER_LIVE_STREAM_STATUS_MODE: EnumRegisterLiveStreamStatus.ValueType
REGISTER_LIVE_STREAM_STATUS_BITRATE: EnumRegisterLiveStreamStatus.ValueType
global___EnumRegisterLiveStreamStatus = EnumRegisterLiveStreamStatus

class _EnumWindowSize:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumWindowSizeEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumWindowSize.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    WINDOW_SIZE_480: _EnumWindowSize.ValueType
    WINDOW_SIZE_720: _EnumWindowSize.ValueType
    WINDOW_SIZE_1080: _EnumWindowSize.ValueType

class EnumWindowSize(_EnumWindowSize, metaclass=_EnumWindowSizeEnumTypeWrapper): ...

WINDOW_SIZE_480: EnumWindowSize.ValueType
WINDOW_SIZE_720: EnumWindowSize.ValueType
WINDOW_SIZE_1080: EnumWindowSize.ValueType
global___EnumWindowSize = EnumWindowSize

@typing_extensions.final
class NotifyLiveStreamStatus(google.protobuf.message.Message):
    """*
    Live Stream status

    Sent either:

      - As a synchronous response to initial @ref RequestGetLiveStreamStatus
      - As an asynchronous notifications registered for via @ref RequestGetLiveStreamStatus
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LIVE_STREAM_STATUS_FIELD_NUMBER: builtins.int
    LIVE_STREAM_ERROR_FIELD_NUMBER: builtins.int
    LIVE_STREAM_ENCODE_FIELD_NUMBER: builtins.int
    LIVE_STREAM_BITRATE_FIELD_NUMBER: builtins.int
    LIVE_STREAM_WINDOW_SIZE_SUPPORTED_ARRAY_FIELD_NUMBER: builtins.int
    LIVE_STREAM_ENCODE_SUPPORTED_FIELD_NUMBER: builtins.int
    LIVE_STREAM_MAX_LENS_UNSUPPORTED_FIELD_NUMBER: builtins.int
    LIVE_STREAM_MINIMUM_STREAM_BITRATE_FIELD_NUMBER: builtins.int
    LIVE_STREAM_MAXIMUM_STREAM_BITRATE_FIELD_NUMBER: builtins.int
    LIVE_STREAM_LENS_SUPPORTED_FIELD_NUMBER: builtins.int
    LIVE_STREAM_LENS_SUPPORTED_ARRAY_FIELD_NUMBER: builtins.int
    LIVE_STREAM_PROTUNE_SUPPORTED_FIELD_NUMBER: builtins.int
    live_stream_status: global___EnumLiveStreamStatus.ValueType
    "Live stream status"
    live_stream_error: global___EnumLiveStreamError.ValueType
    "Live stream error"
    live_stream_encode: builtins.bool
    "Is live stream encoding?"
    live_stream_bitrate: builtins.int
    "Live stream bitrate (Kbps)"

    @property
    def live_stream_window_size_supported_array(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___EnumWindowSize.ValueType]:
        """Set of currently supported resolutions"""
    live_stream_encode_supported: builtins.bool
    "Does the camera support encoding while live streaming?"
    live_stream_max_lens_unsupported: builtins.bool
    "Is the Max Lens feature NOT supported?"
    live_stream_minimum_stream_bitrate: builtins.int
    "Camera-defined minimum bitrate (static) (Kbps)"
    live_stream_maximum_stream_bitrate: builtins.int
    "Camera-defined maximum bitrate (static) (Kbps)"
    live_stream_lens_supported: builtins.bool
    "Does camera support setting lens for live streaming?"

    @property
    def live_stream_lens_supported_array(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___EnumLens.ValueType]:
        """Set of currently supported FOV options"""
    live_stream_protune_supported: builtins.bool
    "Does the camera support live stream protune settings"

    def __init__(
        self,
        *,
        live_stream_status: global___EnumLiveStreamStatus.ValueType | None = ...,
        live_stream_error: global___EnumLiveStreamError.ValueType | None = ...,
        live_stream_encode: builtins.bool | None = ...,
        live_stream_bitrate: builtins.int | None = ...,
        live_stream_window_size_supported_array: (
            collections.abc.Iterable[global___EnumWindowSize.ValueType] | None
        ) = ...,
        live_stream_encode_supported: builtins.bool | None = ...,
        live_stream_max_lens_unsupported: builtins.bool | None = ...,
        live_stream_minimum_stream_bitrate: builtins.int | None = ...,
        live_stream_maximum_stream_bitrate: builtins.int | None = ...,
        live_stream_lens_supported: builtins.bool | None = ...,
        live_stream_lens_supported_array: collections.abc.Iterable[global___EnumLens.ValueType] | None = ...,
        live_stream_protune_supported: builtins.bool | None = ...
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "live_stream_bitrate",
            b"live_stream_bitrate",
            "live_stream_encode",
            b"live_stream_encode",
            "live_stream_encode_supported",
            b"live_stream_encode_supported",
            "live_stream_error",
            b"live_stream_error",
            "live_stream_lens_supported",
            b"live_stream_lens_supported",
            "live_stream_max_lens_unsupported",
            b"live_stream_max_lens_unsupported",
            "live_stream_maximum_stream_bitrate",
            b"live_stream_maximum_stream_bitrate",
            "live_stream_minimum_stream_bitrate",
            b"live_stream_minimum_stream_bitrate",
            "live_stream_protune_supported",
            b"live_stream_protune_supported",
            "live_stream_status",
            b"live_stream_status",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "live_stream_bitrate",
            b"live_stream_bitrate",
            "live_stream_encode",
            b"live_stream_encode",
            "live_stream_encode_supported",
            b"live_stream_encode_supported",
            "live_stream_error",
            b"live_stream_error",
            "live_stream_lens_supported",
            b"live_stream_lens_supported",
            "live_stream_lens_supported_array",
            b"live_stream_lens_supported_array",
            "live_stream_max_lens_unsupported",
            b"live_stream_max_lens_unsupported",
            "live_stream_maximum_stream_bitrate",
            b"live_stream_maximum_stream_bitrate",
            "live_stream_minimum_stream_bitrate",
            b"live_stream_minimum_stream_bitrate",
            "live_stream_protune_supported",
            b"live_stream_protune_supported",
            "live_stream_status",
            b"live_stream_status",
            "live_stream_window_size_supported_array",
            b"live_stream_window_size_supported_array",
        ],
    ) -> None: ...

global___NotifyLiveStreamStatus = NotifyLiveStreamStatus

@typing_extensions.final
class RequestGetLiveStreamStatus(google.protobuf.message.Message):
    """*
    Get the current livestream status (and optionally register for future status changes)

    Response: @ref NotifyLiveStreamStatus

    Notification: @ref NotifyLiveStreamStatus
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REGISTER_LIVE_STREAM_STATUS_FIELD_NUMBER: builtins.int
    UNREGISTER_LIVE_STREAM_STATUS_FIELD_NUMBER: builtins.int

    @property
    def register_live_stream_status(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        global___EnumRegisterLiveStreamStatus.ValueType
    ]:
        """Array of live stream statuses to be notified about"""

    @property
    def unregister_live_stream_status(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        global___EnumRegisterLiveStreamStatus.ValueType
    ]:
        """Array of live stream statuses to stop being notified about"""

    def __init__(
        self,
        *,
        register_live_stream_status: (
            collections.abc.Iterable[global___EnumRegisterLiveStreamStatus.ValueType] | None
        ) = ...,
        unregister_live_stream_status: (
            collections.abc.Iterable[global___EnumRegisterLiveStreamStatus.ValueType] | None
        ) = ...
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "register_live_stream_status",
            b"register_live_stream_status",
            "unregister_live_stream_status",
            b"unregister_live_stream_status",
        ],
    ) -> None: ...

global___RequestGetLiveStreamStatus = RequestGetLiveStreamStatus

@typing_extensions.final
class RequestSetLiveStreamMode(google.protobuf.message.Message):
    """*
    Configure Live Streaming

    Response: @ref ResponseGeneric
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    URL_FIELD_NUMBER: builtins.int
    ENCODE_FIELD_NUMBER: builtins.int
    WINDOW_SIZE_FIELD_NUMBER: builtins.int
    CERT_FIELD_NUMBER: builtins.int
    MINIMUM_BITRATE_FIELD_NUMBER: builtins.int
    MAXIMUM_BITRATE_FIELD_NUMBER: builtins.int
    STARTING_BITRATE_FIELD_NUMBER: builtins.int
    LENS_FIELD_NUMBER: builtins.int
    url: builtins.str
    "RTMP(S) URL used for live stream"
    encode: builtins.bool
    "Save media to sdcard while streaming?"
    window_size: global___EnumWindowSize.ValueType
    "*\n    Resolution to use for live stream\n\n    The set of supported resolutions is only available from the `live_stream_window_size_supported_array`\n    in @ref NotifyLiveStreamStatus.\n    "
    cert: builtins.bytes
    "Certificate for servers that require it in PEM format"
    minimum_bitrate: builtins.int
    "Minimum desired bitrate (may or may not be honored)"
    maximum_bitrate: builtins.int
    "Maximum desired bitrate (may or may not be honored)"
    starting_bitrate: builtins.int
    "Starting bitrate"
    lens: global___EnumLens.ValueType
    "*\n    Lens to use for live stream\n\n    The set of supported lenses is only available from the  `live_stream_lens_supported_array`\n    in @ref NotifyLiveStreamStatus.\n    "

    def __init__(
        self,
        *,
        url: builtins.str | None = ...,
        encode: builtins.bool | None = ...,
        window_size: global___EnumWindowSize.ValueType | None = ...,
        cert: builtins.bytes | None = ...,
        minimum_bitrate: builtins.int | None = ...,
        maximum_bitrate: builtins.int | None = ...,
        starting_bitrate: builtins.int | None = ...,
        lens: global___EnumLens.ValueType | None = ...
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "cert",
            b"cert",
            "encode",
            b"encode",
            "lens",
            b"lens",
            "maximum_bitrate",
            b"maximum_bitrate",
            "minimum_bitrate",
            b"minimum_bitrate",
            "starting_bitrate",
            b"starting_bitrate",
            "url",
            b"url",
            "window_size",
            b"window_size",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "cert",
            b"cert",
            "encode",
            b"encode",
            "lens",
            b"lens",
            "maximum_bitrate",
            b"maximum_bitrate",
            "minimum_bitrate",
            b"minimum_bitrate",
            "starting_bitrate",
            b"starting_bitrate",
            "url",
            b"url",
            "window_size",
            b"window_size",
        ],
    ) -> None: ...

global___RequestSetLiveStreamMode = RequestSetLiveStreamMode



# http_settings.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb 20 23:24:52 UTC 2025

"""HTTP Settings"""

########################################################################################################################
#
# Warning!! This file is auto-generated. Do not modify it manually.
#
########################################################################################################################

from open_gopro import models
from open_gopro.api.builders import HttpSetting
from open_gopro.domain.communicator_interface import GoProHttp, HttpMessages
from open_gopro.models.constants import SettingId, settings


class HttpSettings(HttpMessages[HttpSetting]):
    # pylint: disable=missing-class-docstring, unused-argument
    """The collection of all HTTP Settings

    Args:
        communicator (GoProHttp): Adapter to read / write settings
    """

    def __init__(self, communicator: GoProHttp):

        self.video_resolution: HttpSetting[settings.VideoResolution] = HttpSetting[settings.VideoResolution](
            communicator, SettingId.VIDEO_RESOLUTION
        )

        """Video Resolution

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-resolution-2)"""

        self.frames_per_second: HttpSetting[settings.FramesPerSecond] = HttpSetting[settings.FramesPerSecond](
            communicator, SettingId.FRAMES_PER_SECOND
        )

        """Frames Per Second

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#frames-per-second-3)"""

        self.video_timelapse_rate: HttpSetting[settings.VideoTimelapseRate] = HttpSetting[settings.VideoTimelapseRate](
            communicator, SettingId.VIDEO_TIMELAPSE_RATE
        )

        """Video Timelapse Rate

        How frequently to take a video when performing a Video Timelapse

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-timelapse-rate-5)"""

        self.photo_timelapse_rate: HttpSetting[settings.PhotoTimelapseRate] = HttpSetting[settings.PhotoTimelapseRate](
            communicator, SettingId.PHOTO_TIMELAPSE_RATE
        )

        """Photo Timelapse Rate

        How frequently to take a photo when performing a Photo Timelapse.

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-timelapse-rate-30)"""

        self.nightlapse_rate: HttpSetting[settings.NightlapseRate] = HttpSetting[settings.NightlapseRate](
            communicator, SettingId.NIGHTLAPSE_RATE
        )

        """Nightlapse Rate

        How frequently to take a video or photo when performing a Nightlapse.

		This controls the Video or Photo Nightlapse rate if Setting 128 is set to 21 or 26 respectively.

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#nightlapse-rate-32)"""

        self.webcam_digital_lenses: HttpSetting[settings.WebcamDigitalLenses] = HttpSetting[
            settings.WebcamDigitalLenses
        ](communicator, SettingId.WEBCAM_DIGITAL_LENSES)

        """Webcam Digital Lenses

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#webcam-digital-lenses-43)"""

        self.auto_power_down: HttpSetting[settings.AutoPowerDown] = HttpSetting[settings.AutoPowerDown](
            communicator, SettingId.AUTO_POWER_DOWN
        )

        """Auto Power Down

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#auto-power-down-59)"""

        self.gps: HttpSetting[settings.Gps] = HttpSetting[settings.Gps](communicator, SettingId.GPS)

        """GPS

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#gps-83)"""

        self.lcd_brightness: HttpSetting[int] = HttpSetting[int](communicator, SettingId.LCD_BRIGHTNESS)

        """LCD Brightness

        The LCD brightness as a percentage value from 10-100

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#lcd-brightness-88)"""

        self.led: HttpSetting[settings.Led] = HttpSetting[settings.Led](communicator, SettingId.LED)

        """LED

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#led-91)"""

        self.video_aspect_ratio: HttpSetting[settings.VideoAspectRatio] = HttpSetting[settings.VideoAspectRatio](
            communicator, SettingId.VIDEO_ASPECT_RATIO
        )

        """Video Aspect Ratio

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-aspect-ratio-108)"""

        self.video_lens: HttpSetting[settings.VideoLens] = HttpSetting[settings.VideoLens](
            communicator, SettingId.VIDEO_LENS
        )

        """Video Lens

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-lens-121)"""

        self.photo_lens: HttpSetting[settings.PhotoLens] = HttpSetting[settings.PhotoLens](
            communicator, SettingId.PHOTO_LENS
        )

        """Photo Lens

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-lens-122)"""

        self.time_lapse_digital_lenses: HttpSetting[settings.TimeLapseDigitalLenses] = HttpSetting[
            settings.TimeLapseDigitalLenses
        ](communicator, SettingId.TIME_LAPSE_DIGITAL_LENSES)

        """Time Lapse Digital Lenses

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#time-lapse-digital-lenses-123)"""

        self.photo_output: HttpSetting[settings.PhotoOutput] = HttpSetting[settings.PhotoOutput](
            communicator, SettingId.PHOTO_OUTPUT
        )

        """Photo Output

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-output-125)"""

        self.media_format: HttpSetting[settings.MediaFormat] = HttpSetting[settings.MediaFormat](
            communicator, SettingId.MEDIA_FORMAT
        )

        """Media Format

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#media-format-128)"""

        self.anti_flicker: HttpSetting[settings.Anti_Flicker] = HttpSetting[settings.Anti_Flicker](
            communicator, SettingId.ANTI_FLICKER
        )

        """Anti-Flicker

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#anti-flicker-134)"""

        self.hypersmooth: HttpSetting[settings.Hypersmooth] = HttpSetting[settings.Hypersmooth](
            communicator, SettingId.HYPERSMOOTH
        )

        """Hypersmooth

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#hypersmooth-135)"""

        self.video_horizon_leveling: HttpSetting[settings.VideoHorizonLeveling] = HttpSetting[
            settings.VideoHorizonLeveling
        ](communicator, SettingId.VIDEO_HORIZON_LEVELING)

        """Video Horizon Leveling

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-horizon-leveling-150)"""

        self.photo_horizon_leveling: HttpSetting[settings.PhotoHorizonLeveling] = HttpSetting[
            settings.PhotoHorizonLeveling
        ](communicator, SettingId.PHOTO_HORIZON_LEVELING)

        """Photo Horizon Leveling

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-horizon-leveling-151)"""

        self.video_duration: HttpSetting[settings.VideoDuration] = HttpSetting[settings.VideoDuration](
            communicator, SettingId.VIDEO_DURATION
        )

        """Video Duration

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-duration-156)"""

        self.multi_shot_duration: HttpSetting[settings.MultiShotDuration] = HttpSetting[settings.MultiShotDuration](
            communicator, SettingId.MULTI_SHOT_DURATION
        )

        """Multi Shot Duration

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#multi-shot-duration-157)"""

        self.max_lens: HttpSetting[settings.MaxLens] = HttpSetting[settings.MaxLens](communicator, SettingId.MAX_LENS)

        """Max Lens

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#max-lens-162)"""

        self.hindsight: HttpSetting[settings.Hindsight] = HttpSetting[settings.Hindsight](
            communicator, SettingId.HINDSIGHT
        )

        """HindSight

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#hindsight-167)"""

        self.scheduled_capture: HttpSetting[models.ScheduledCapture] = HttpSetting[models.ScheduledCapture](
            communicator, SettingId.SCHEDULED_CAPTURE
        )

        """Scheduled Capture

        Configure or disable the scheduled capture functionality to start encoding at a future time.

		This is a bit-masked value with the following bitwise definitions,
		numbered from least significant bit, with the example showing parsed fields from a sample of `0x00000c8b`.

		| Bit(s) | Definition                    | Example |
		| ------ | ----------------------------- | ------- |
		| 0      | Is Scheduled Capture Enabled? | 1       |
		| 1      | Is 24 hour format?            | 1       |
		| 2-8    | Minute                        | 34      |
		| 9-13   | Hour                          | 12      |
		| 14-63  | Reserved                      | 0       |

		Note that when the scheduled capture time occurs, encoding will be started and continue indefinitely.  One
		of the duration settings (156, 157, 172) can be used to set the encoding duration depending on the camera mode.

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#scheduled-capture-168)"""

        self.photo_single_interval: HttpSetting[settings.PhotoSingleInterval] = HttpSetting[
            settings.PhotoSingleInterval
        ](communicator, SettingId.PHOTO_SINGLE_INTERVAL)

        """Photo Single Interval

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-single-interval-171)"""

        self.photo_interval_duration: HttpSetting[settings.PhotoIntervalDuration] = HttpSetting[
            settings.PhotoIntervalDuration
        ](communicator, SettingId.PHOTO_INTERVAL_DURATION)

        """Photo Interval Duration

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-interval-duration-172)"""

        self.video_performance_mode: HttpSetting[settings.VideoPerformanceMode] = HttpSetting[
            settings.VideoPerformanceMode
        ](communicator, SettingId.VIDEO_PERFORMANCE_MODE)

        """Video Performance Mode

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-performance-mode-173)"""

        self.control_mode: HttpSetting[settings.ControlMode] = HttpSetting[settings.ControlMode](
            communicator, SettingId.CONTROL_MODE
        )

        """Control Mode

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#control-mode-175)"""

        self.easy_mode_speed: HttpSetting[settings.EasyModeSpeed] = HttpSetting[settings.EasyModeSpeed](
            communicator, SettingId.EASY_MODE_SPEED
        )

        """Easy Mode Speed

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#easy-mode-speed-176)"""

        self.enable_night_photo: HttpSetting[settings.EnableNightPhoto] = HttpSetting[settings.EnableNightPhoto](
            communicator, SettingId.ENABLE_NIGHT_PHOTO
        )

        """Enable Night Photo

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#enable-night-photo-177)"""

        self.wireless_band: HttpSetting[settings.WirelessBand] = HttpSetting[settings.WirelessBand](
            communicator, SettingId.WIRELESS_BAND
        )

        """Wireless Band

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#wireless-band-178)"""

        self.star_trails_length: HttpSetting[settings.StarTrailsLength] = HttpSetting[settings.StarTrailsLength](
            communicator, SettingId.STAR_TRAILS_LENGTH
        )

        """Star Trails Length

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#star-trails-length-179)"""

        self.system_video_mode: HttpSetting[settings.SystemVideoMode] = HttpSetting[settings.SystemVideoMode](
            communicator, SettingId.SYSTEM_VIDEO_MODE
        )

        """System Video Mode

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#system-video-mode-180)"""

        self.video_bit_rate: HttpSetting[settings.VideoBitRate] = HttpSetting[settings.VideoBitRate](
            communicator, SettingId.VIDEO_BIT_RATE
        )

        """Video Bit Rate

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-bit-rate-182)"""

        self.bit_depth: HttpSetting[settings.BitDepth] = HttpSetting[settings.BitDepth](
            communicator, SettingId.BIT_DEPTH
        )

        """Bit Depth

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#bit-depth-183)"""

        self.profiles: HttpSetting[settings.Profiles] = HttpSetting[settings.Profiles](communicator, SettingId.PROFILES)

        """Profiles

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#profiles-184)"""

        self.video_easy_mode: HttpSetting[settings.VideoEasyMode] = HttpSetting[settings.VideoEasyMode](
            communicator, SettingId.VIDEO_EASY_MODE
        )

        """Video Easy Mode

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-easy-mode-186)"""

        self.lapse_mode: HttpSetting[settings.LapseMode] = HttpSetting[settings.LapseMode](
            communicator, SettingId.LAPSE_MODE
        )

        """Lapse Mode

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#lapse-mode-187)"""

        self.max_lens_mod: HttpSetting[settings.MaxLensMod] = HttpSetting[settings.MaxLensMod](
            communicator, SettingId.MAX_LENS_MOD
        )

        """Max Lens Mod

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#max-lens-mod-189)"""

        self.max_lens_mod_enable: HttpSetting[settings.MaxLensModEnable] = HttpSetting[settings.MaxLensModEnable](
            communicator, SettingId.MAX_LENS_MOD_ENABLE
        )

        """Max Lens Mod Enable

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#max-lens-mod-enable-190)"""

        self.easy_night_photo: HttpSetting[settings.EasyNightPhoto] = HttpSetting[settings.EasyNightPhoto](
            communicator, SettingId.EASY_NIGHT_PHOTO
        )

        """Easy Night Photo

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#easy-night-photo-191)"""

        self.multi_shot_aspect_ratio: HttpSetting[settings.MultiShotAspectRatio] = HttpSetting[
            settings.MultiShotAspectRatio
        ](communicator, SettingId.MULTI_SHOT_ASPECT_RATIO)

        """Multi Shot Aspect Ratio

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#multi-shot-aspect-ratio-192)"""

        self.framing: HttpSetting[settings.Framing] = HttpSetting[settings.Framing](communicator, SettingId.FRAMING)

        """Framing

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#framing-193)"""

        self.camera_volume: HttpSetting[settings.CameraVolume] = HttpSetting[settings.CameraVolume](
            communicator, SettingId.CAMERA_VOLUME
        )

        """Camera Volume

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#camera-volume-216)"""

        self.setup_screen_saver: HttpSetting[settings.SetupScreenSaver] = HttpSetting[settings.SetupScreenSaver](
            communicator, SettingId.SETUP_SCREEN_SAVER
        )

        """Setup Screen Saver

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#setup-screen-saver-219)"""

        self.setup_language: HttpSetting[settings.SetupLanguage] = HttpSetting[settings.SetupLanguage](
            communicator, SettingId.SETUP_LANGUAGE
        )

        """Setup Language

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#setup-language-223)"""

        self.photo_mode: HttpSetting[settings.PhotoMode] = HttpSetting[settings.PhotoMode](
            communicator, SettingId.PHOTO_MODE
        )

        """Photo Mode

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#photo-mode-227)"""

        self.video_framing: HttpSetting[settings.VideoFraming] = HttpSetting[settings.VideoFraming](
            communicator, SettingId.VIDEO_FRAMING
        )

        """Video Framing

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#video-framing-232)"""

        self.multi_shot_framing: HttpSetting[settings.MultiShotFraming] = HttpSetting[settings.MultiShotFraming](
            communicator, SettingId.MULTI_SHOT_FRAMING
        )

        """Multi Shot Framing

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#multi-shot-framing-233)"""

        self.frame_rate: HttpSetting[settings.FrameRate] = HttpSetting[settings.FrameRate](
            communicator, SettingId.FRAME_RATE
        )

        """Frame Rate

        @see [Open GoPro Spec](https://gopro.github.io/OpenGoPro/ble/features/settings.html#frame-rate-234)"""

        super().__init__(communicator)



# scanner.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Tue Mar 25 16:44:35 UTC 2025

"""Demo to retrieve and parse information from non-connected GoPros"""

import argparse
import asyncio
import enum
import sys
from abc import ABC
from dataclasses import dataclass
from typing import Generic, TypeVar

from bleak import BleakScanner
from rich.console import Console

from open_gopro.domain.exceptions import FailedToFindDevice
from open_gopro.gopro_wired import WiredGoPro
from open_gopro.models import AdvData, GoProAdvData
from open_gopro.network.wifi.mdns_scanner import find_first_ip_addr
from open_gopro.util import add_cli_args_and_parse, ainput

console = Console()


class NetworkType(enum.Enum):
    """Scan Response Network type"""

    BLE = enum.auto()
    WIFI_AP = enum.auto()
    DNS = enum.auto()


T = TypeVar("T")


@dataclass(frozen=True)
class ScanResponse(ABC, Generic[T]):
    """Base class for scan responses"""

    data: T
    network: NetworkType


@dataclass(frozen=True)
class BleScanResponse(ScanResponse[GoProAdvData]):
    """A BLE advertisement / scan response"""

    data: GoProAdvData
    network: NetworkType = NetworkType.BLE


@dataclass(frozen=True)
class WifiApScanResponse(ScanResponse[str]):
    """An SSID discovered by scanning via WiFi"""

    data: str
    network: NetworkType = NetworkType.WIFI_AP


@dataclass(frozen=True)
class DnsScanResponse(ScanResponse[str]):
    """ "A DNS entry discovered by scanning mDNS"""

    data: str
    network: NetworkType = NetworkType.DNS


def process_scan_response(scan_response: ScanResponse) -> None:
    """Log the scan response

    Args:
        scan_response (ScanResponse): scan response to process
    """
    match (scan_response):
        case BleScanResponse():
            console.print(f"{scan_response.network.name} ==> {scan_response.data.serial_number}")
        case WifiApScanResponse() | DnsScanResponse():
            console.print(f"{scan_response.network.name} ==> {scan_response.data}")


async def scan_ble() -> None:
    """Scan BLE Advertisements / scan responses and combine them to find GoPros"""
    unique_adv_data: set[GoProAdvData] = set()

    async with BleakScanner(service_uuids=["0000fea6-0000-1000-8000-00805f9b34fb"]) as scanner:
        while True:
            adv_data = AdvData()
            async for _, data in scanner.advertisement_data():
                adv_data.update(data)
                if adv_data.local_name:  # Once we've received the scan response...
                    break

            if (gopro_adv_data := GoProAdvData.fromAdvData(adv_data)) not in unique_adv_data:
                process_scan_response(BleScanResponse(gopro_adv_data))
            unique_adv_data.add(gopro_adv_data)


async def scan_wifi_ap() -> None:
    """Scan for GoPro XXXX WiFi SSIDs"""
    raise NotImplementedError


async def scan_dns() -> None:
    """Scan for MDNS entries that match the GoPro service name"""
    unique_responses: set[str] = set()

    while True:
        try:
            response = await find_first_ip_addr(WiredGoPro._MDNS_SERVICE_NAME, 10)
            serial = response.name.split(".")[0]
            if serial not in unique_responses:
                process_scan_response(DnsScanResponse(serial))
            unique_responses.add(serial)
        except FailedToFindDevice:
            continue


async def wait_for_exit() -> None:
    """Exit on enter"""
    await ainput("Press enter to exit\n")
    console.print("Exiting...")
    sys.exit()


async def main(_: argparse.Namespace) -> None:
    await asyncio.gather(
        scan_ble(),
        # scan_wifi_ap(args.password, args.interface),
        scan_dns(),
        wait_for_exit(),
    )


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description="Scan for GoPro's via BLE and mDNS.")

    return add_cli_args_and_parse(parser, bluetooth=False)


# Needed for poetry scripts defined in pyproject.toml
def entrypoint() -> None:
    asyncio.run(main(parse_arguments()))


if __name__ == "__main__":
    entrypoint()



# json.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Apr 21 22:24:00 UTC 2025

"""JSON Parser and Transformer implementations"""

from __future__ import annotations

import logging
from typing import Callable, TypeVar, cast

from construct import FormatFieldError
from pydantic import BaseModel

from open_gopro.domain.communicator_interface import GlobalParsers
from open_gopro.domain.parser_interface import JsonParser, JsonTransformer
from open_gopro.domain.types import CameraState, JsonDict, ResponseType
from open_gopro.models.constants import SettingId, StatusId
from open_gopro.util import map_keys

T = TypeVar("T")

logger = logging.getLogger()


class CameraStateJsonParser(JsonParser):
    """Parse integer numbers into Enums"""

    def parse(self, data: JsonDict) -> CameraState:
        """Parse dict of integer values into human readable (i.e. enum'ed) setting / status map

        Args:
            data (JsonDict): input dict to parse

        Returns:
            CameraState: output human readable dict
        """
        parsed: dict = {}
        # Parse status and settings values into nice human readable things
        for name, id_map in [("status", StatusId), ("settings", SettingId)]:
            for k, v in data[name].items():
                try:
                    identifier = cast(ResponseType, id_map(int(k)))
                    if not (parser_builder := GlobalParsers.get_query_container(identifier)):
                        parsed[identifier] = v
                    else:
                        parsed[identifier] = parser_builder(v)
                except (ValueError, FormatFieldError) as e:
                    logger.trace(f"Error Parsing {name}::{k}, value: {v} ==> {repr(e)}")  # type: ignore
                    continue
        return parsed


class PydanticAdapterJsonParser(JsonParser[BaseModel]):
    """Parse Json using a Pydantic model

    Args:
        model (type[BaseModel]): model to use for parsing
    """

    def __init__(self, model: type[BaseModel]) -> None:
        self.model = model

    def parse(self, data: JsonDict) -> BaseModel:
        """Parse json dict into model

        Args:
            data (JsonDict): data to parse

        Returns:
            BaseModel: parsed model
        """
        return self.model(**data)


class LambdaJsonParser(JsonParser[T]):
    """Helper class to allow parser definition using a lambda

    Args:
        parser (Callable[[JsonDict], T]): lambda to parse input
    """

    def __init__(self, parser: Callable[[JsonDict], T]) -> None:
        self._parser = parser

    def parse(self, data: JsonDict) -> T:
        """Use stored lambda parse for parsing

        Args:
            data (JsonDict): input dict to parse

        Returns:
            T: parsed output
        """
        return self._parser(data)


class MapJsonKey(JsonTransformer):
    """Map all matching keys using the input function"""

    def __init__(self, key: str, func: Callable) -> None:
        self.key = key
        self.func = func
        super().__init__()

    def transform(self, data: JsonDict) -> JsonDict:
        """Transform json, mapping keys

        Args:
            data (JsonDict): json data to transform

        Returns:
            JsonDict: transformed json data
        """
        map_keys(data, self.key, self.func)
        return data



# ble_advertisement.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Nov 18 21:03:37 UTC 2024

"""GoPro specific advertisement entities and parsing structures"""

from __future__ import annotations

import json
import logging
from dataclasses import asdict, dataclass, field
from typing import Any

from bleak.backends.scanner import AdvertisementData
from construct import (
    Adapter,
    BitStruct,
    Byte,
    Bytes,
    Flag,
    GreedyString,
    Hex,
    Int16ub,
    Int16ul,
    PaddedString,
    Padding,
    Struct,
    this,
)

from open_gopro.util import deeply_update_dict

logger = logging.getLogger(__name__)


class Hexlify(Adapter):
    """Construct adapter for pretty hex representation"""

    def _decode(self, obj: bytes, context: Any, path: Any) -> str:
        return obj.hex(":")

    def _encode(self, obj: str, context: Any, path: Any) -> list[int]:
        return list(map(int, obj.split(":")))


camera_status_struct = BitStruct(
    "processor_state" / Flag,
    "wifi_ap_state" / Flag,
    "peripheral_pairing_state" / Flag,
    "central_role_enabled" / Flag,
    "is_new_media_available" / Flag,
    "reserved" / Padding(3),
)


camera_capability_struct = BitStruct(
    "cnc" / Flag,
    "ble_metadata" / Flag,
    "wideband_audio" / Flag,
    "concurrent_master_slave" / Flag,
    "onboarding" / Flag,
    "new_media_available" / Flag,
    "reserved" / Padding(10),
)

media_offload_status_struct = BitStruct(
    "available" / Flag,
    "new_media_available" / Flag,
    "battery_ok" / Flag,
    "sd_card_ok" / Flag,
    "busy" / Flag,
    "paused" / Flag,
    "reserved" / Padding(2),
)

manuf_data_struct = Struct(
    "schema_version" / Byte,
    "camera_status" / camera_status_struct,
    "camera_id" / Byte,
    "camera_capabilities" / camera_capability_struct,
    "id_hash" / Bytes(6),
    "media_offload_status" / media_offload_status_struct,
)

adv_data_struct = Struct(
    "flags_length" / Byte,
    "flags" / Hex(Int16ub),
    "uuids_length" / Byte,
    "uuids_type" / Hex(Byte),
    "uuids" / Hex(Int16ul),
    "manuf_length" / Byte,
    "manuf_type" / Hex(Byte),
    "company_id" / Hex(Int16ub),
    "manuf_data" / manuf_data_struct,
)

service_data_struct = Struct(
    "ap_mac_address" / Hexlify(Bytes(4)),
    "serial_number" / GreedyString("utf8"),
)

scan_response_struct = Struct(
    "name_length" / Byte,
    "name_type" / Hex(Byte),
    "name" / PaddedString(this.name_length - 1, "utf8"),
    "service_length" / Byte,
    "service_type" / Hex(Byte),
    "service_uuid" / Hex(Int16ul),
    "service_data" / service_data_struct,
)


@dataclass
class Jsonable:
    """Mixin to use pretty hex presentation for JSON decoding"""

    def __str__(self) -> str:
        def default_decode(obj: Any) -> Any:
            if isinstance(obj, (bytes, bytearray)):
                return obj.hex(":")
            return str(obj)

        return json.dumps(asdict(self), indent=4, default=default_decode)


camera_id_map: dict[int, str] = {
    55: "C344",
    57: "C346",
    58: "C347",
    60: "C349",
    62: "C350",
    64: "C352",
    65: "C353",
}


@dataclass
class GoProAdvData(Jsonable):
    """GoPro-specific advertising data"""

    name: str
    schema_version: int
    processor_state: bool
    wifi_ap_state: bool
    peripheral_pairing_state: bool
    is_new_media_available: bool
    camera_id: int
    supports_cnc: bool
    supports_ble_metadata: bool
    supports_wideband_audio: bool
    supports_concurrent_master_slave: bool
    supports_onboarding: bool
    supports_new_media_available: bool
    id_hash: bytes
    is_media_upload_new_media_available: bool
    is_media_upload_available: bool
    is_media_upload_battery_ok: bool
    is_media_upload_sd_card_ok: bool
    is_media_upload_busy: bool
    is_media_upload_paused: bool
    ap_mac_address: bytes
    partial_serial_number: str

    def __hash__(self) -> int:
        return hash(self.serial_number)

    @property
    def serial_number(self) -> str:
        """Get the serial number as accurately as possible

        Returns:
            str: serial number with X's denoting unknown characters
        """
        try:
            prefix_4 = camera_id_map[self.camera_id]
        except KeyError:
            logger.warning(f"Unknown camera ID {self.camera_id}")
            prefix_4 = "XXXX"
        try:
            middle_6 = self.id_hash.decode("utf-8")
        except UnicodeDecodeError:
            middle_6 = "XXXXXX"
        return f"{prefix_4}{middle_6}{self.partial_serial_number}"

    @classmethod
    def fromAdvData(cls, data: AdvData) -> GoProAdvData:
        """Build GoPro specific advertisement from standard BLE advertisement data

        Args:
            data (AdvData): standard BLE advertisement data

        Returns:
            GoProAdvData: parsed GoPro specific advertising data
        """
        manuf_data = manuf_data_struct.parse(list(data.manufacturer_data.values())[0])
        service_data = service_data_struct.parse(list(data.service_data.values())[0])
        return GoProAdvData(
            # Name from scan response data
            name=data.local_name,
            # Schema version from advertising data manufacturer data
            schema_version=manuf_data.schema_version,
            # Camera status from advertising data manufacturer data
            processor_state=manuf_data.camera_status.processor_state,
            wifi_ap_state=manuf_data.camera_status.wifi_ap_state,
            peripheral_pairing_state=manuf_data.camera_status.peripheral_pairing_state,
            is_new_media_available=manuf_data.camera_status.is_new_media_available,
            # Camera ID from advertising data manufacturer data
            camera_id=int(manuf_data.camera_id),
            # Camera capabilities from advertising data manufacturer data
            supports_ble_metadata=manuf_data.camera_capabilities.ble_metadata,
            supports_cnc=manuf_data.camera_capabilities.cnc,
            supports_onboarding=manuf_data.camera_capabilities.onboarding,
            supports_wideband_audio=manuf_data.camera_capabilities.wideband_audio,
            supports_concurrent_master_slave=manuf_data.camera_capabilities.concurrent_master_slave,
            supports_new_media_available=manuf_data.camera_capabilities.new_media_available,
            # ID Hash from advertising data manufacturer's data
            id_hash=manuf_data.id_hash,
            # Media offload status status from advertising data manufacturer's data
            is_media_upload_new_media_available=manuf_data.media_offload_status.new_media_available,
            is_media_upload_available=manuf_data.media_offload_status.available,
            is_media_upload_battery_ok=manuf_data.media_offload_status.battery_ok,
            is_media_upload_sd_card_ok=manuf_data.media_offload_status.sd_card_ok,
            is_media_upload_busy=manuf_data.media_offload_status.busy,
            is_media_upload_paused=manuf_data.media_offload_status.paused,
            # Mac address from scan response data service data
            ap_mac_address=service_data.ap_mac_address,
            # Partial serial number from scan response data service data
            partial_serial_number=service_data.serial_number,
        )


@dataclass
class AdvData(Jsonable):
    """Standard BLE advertising data

    Only contains fields that are currently used by GoPro
    """

    serial_number: str = ""
    local_name: str = ""
    manufacturer_data: dict[str, Any] = field(default_factory=dict)
    service_uuids: list[str] = field(default_factory=list)
    service_data: dict = field(default_factory=dict)

    def update(self, data: AdvertisementData) -> None:
        """Update with a (potentially incomplete) advertisement

        Args:
            data (AdvertisementData): advertisement to use for updating
        """
        self_dict = asdict(self)
        for k, v in data._asdict().items():
            if not v:
                continue
            if isinstance(v, dict):
                setattr(self, k, deeply_update_dict(self_dict[k], v))
            elif isinstance(v, list):
                setattr(self, k, [*self_dict[k], v])
            else:
                setattr(self, k, v)


@dataclass
class DnsScanResponse:
    """mDNS scan data"""

    service: str
    ip_addr: str
    name: str



# observable.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Observable / observer async generators to process asynchronous data stream.

An Observable is a source of asynchronous data that can be observed by multiple observers. Observers can
subscribe to the observable to retrieve an async generator observe and asynchronously receive updates when new data is
emitted. The observer can also perform actions on the data as it is emitted, such as filtering or transforming the data
using the [asyncstdlib](https://pypi.org/project/asyncstdlib) library to manipulate the async generator.
"""

# pylint: disable=redefined-builtin

from __future__ import annotations

import asyncio
import logging
from dataclasses import dataclass, field
from inspect import iscoroutinefunction
from typing import (
    Any,
    AsyncGenerator,
    Callable,
    Coroutine,
    Final,
    Generic,
    NoReturn,
    Self,
    TypeAlias,
    TypeVar,
)
from uuid import UUID, uuid1

from asyncstdlib import anext, filter

O = TypeVar("O")
T = TypeVar("T")


SyncAction: TypeAlias = Callable[[T], None]
AsyncAction: TypeAlias = Callable[[T], Coroutine[Any, Any, None]]
SyncFilter: TypeAlias = Callable[[T], bool]
AsyncFilter: TypeAlias = Callable[[T], Coroutine[Any, Any, bool]]

logger = logging.getLogger(__name__)

T_I = TypeVar("T_I")


class Observer(AsyncGenerator[T, None]):
    """Async generator wrapper with added control methods"""

    def __init__(self, observable: Observable[T], uuid: UUID, replay: int, debug_id: str | None = None) -> None:
        self._observable = observable
        self._uuid = uuid
        self._replay = replay
        self._debug_id = debug_id or str(uuid)
        self._is_active = False

    def __aiter__(self) -> Observer[T]:
        return self

    async def __anext__(self) -> T:
        if not self._is_active:
            self._is_active = True
            await self._observable._add_observer(self._uuid, replay=self._replay)

        try:
            logger.trace(f"Observer {self._debug_id} waiting for next value")  # type: ignore
            value = await self._observable._get_next(self._uuid)
            logger.trace(f"Observer {self._debug_id} received value: {value}")  # type: ignore
            return value
        except Exception as e:
            logger.error(f"Error in observer {self._debug_id}: {e}")
            await self._cleanup()
            raise e

    async def first(self, predicate: SyncFilter) -> T:
        """Get the first value that matches the predicate

        Args:
            predicate (SyncFilter): Predicate to match

        Returns:
            T: First value that matches the predicate
        """
        return await anext(filter(predicate, self))

    async def _cleanup(self) -> None:
        """Clean up resources when generator is done"""
        if self._is_active:
            self._is_active = False
            await self._observable._remove_observer(self._uuid)

    async def aclose(self) -> None:
        """Properly close the generator and clean up resources"""
        await self._cleanup()

    async def athrow(self, typ: Any, val: Any = None, tb: Any = None) -> NoReturn:
        """Throw an exception into the generator"""
        if not self._is_active:
            raise StopAsyncIteration

        # Cleanup first
        await self._cleanup()
        # Then raise the exception
        if val is None:
            val = typ()
        if tb is not None:
            raise val.with_traceback(tb)
        raise val

    async def asend(self, value: None) -> T:
        """Send a value into the generator (required by protocol)"""
        if not self._is_active:
            raise StopAsyncIteration

        # We don't really use the sent value, so just advance to next item
        return await anext(self)


class Observable(Generic[T]):
    """The source of asynchronous data

    Attributes:
        REPLAY_ALL (Final[int]): Special integer value to indicate all values should be replayed
        OBS_IDX (int): counter of observable instantiations used for debugging

    Args:
        capacity (int): Maximum values to store for replay. Defaults to 100.
        debug_id (str | None): Identifier to log for debugging. Defaults to None (will use generated UUID).
    """

    REPLAY_ALL: Final[int] = -1
    OBS_IDX: int = 0

    @dataclass
    class _SharedData(Generic[T_I]):
        """Common data used for internal management that should be accessed in critical sections"""

        current: T_I | None = None
        cache: list[T_I] = field(default_factory=list)
        q_dict: dict[UUID, asyncio.Queue[T_I]] = field(default_factory=dict)

        def __post_init__(self) -> None:  # noqa
            self._condition = asyncio.Condition()

        async def __aenter__(self) -> Observable._SharedData[T_I]:  # noqa
            await self._condition.acquire()
            return self

        async def __aexit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:  # noqa
            self._condition.release()

    def __init__(self, capacity: int = 100, debug_id: str | None = None) -> None:
        self._lock = asyncio.Condition()
        self._count = 0
        self._capacity = capacity
        self._debug_id = debug_id or str(Observable.OBS_IDX)
        Observable.OBS_IDX += 1
        self._on_start_actions: list[SyncAction[T] | AsyncAction[T]] = []
        self._on_subscribe_actions: list[Callable[[], None] | Callable[[], Coroutine[Any, Any, None]]] = []
        self._per_value_actions: list[SyncAction[T] | AsyncAction[T]] = []
        self._shared_data = Observable._SharedData[T]()
        # TODO handle cleanup

    async def _add_observer(self, uuid: UUID, replay: int) -> None:
        """Add an observer to receive collected values

        Args:
            uuid (UUID): observer identifier
            replay (int): how many values to replay from cache
        """
        async with self._shared_data:
            if uuid not in self._shared_data.q_dict:
                self._shared_data.q_dict[uuid] = asyncio.Queue()
                if replay == Observable.REPLAY_ALL:
                    replay = len(self._shared_data.cache)
                head = max(len(self._shared_data.cache) - replay, 0)
                for value in self._shared_data.cache[head:]:
                    self._shared_data.q_dict[uuid].put_nowait(value)

    async def _remove_observer(self, uuid: UUID) -> None:
        """Remove an observer from receiving collected values

        Args:
            uuid (UUID): observer identifier
        """
        async with self._shared_data:
            if uuid in self._shared_data.q_dict:
                del self._shared_data.q_dict[uuid]

    async def emit(self, value: T) -> None:
        """Receive a value and queue it for per-observer retrieval

        Not intended to be used by the observer.

        Args:
            value (T): Value to queue
        """
        async with self._shared_data:
            self._shared_data.current = value
            self._shared_data.cache.append(value)
            if len(self._shared_data.cache) > self._capacity:
                self._shared_data.cache.pop(0)
            for uuid, q in self._shared_data.q_dict.items():
                logger.trace(f"Observable {self._debug_id} emitting {value} to observer {uuid}")  # type: ignore
                await q.put(value)

    def _mux_action(
        self,
        action: SyncAction[T] | AsyncAction[T],
        value: T,
        tg: asyncio.TaskGroup | None = None,
    ) -> None:
        """Execute an action that is either synchronous or asynchronous

        If tg is passed, the async action will be added to the task group. Otherwise an anonymous async task will be
        created. In both cases, this function will return without awaiting the created task.

        Note! If action is synchronous, this will block until the action returns.

        Args:
            action (SyncAction[T] | AsyncAction[T]): action to execute
            value (T): value to pass to action
            tg (asyncio.TaskGroup | None, optional): Task group to add async action. Defaults to None (don't add to any
                task group).
        """
        if iscoroutinefunction(action):
            if tg:
                tg.create_task(action(value))
            else:
                asyncio.create_task(action(value))
        else:
            action(value)

    async def _mux_filter_blocking(self, predicate: SyncFilter | AsyncFilter, value: T) -> bool:
        """Execute a filter that is either synchronous or asynchronous

        Note! This will await / block until the action completes.

        Args:
            predicate (SyncFilter | AsyncFilter): Filter to execute
            value (T): value to analyze with predicates

        Returns:
            bool: _description_
        """
        if iscoroutinefunction(predicate):
            return await predicate(value)
        return predicate(value)  # type: ignore

    @property
    def current(self) -> T | None:
        """Get the most recently collected value of the observable.

        Note that this does not indicate the value in real-time. It is the most recent value that was collected
        from a receiver.

        Returns:
            T | None: Most recently collected value, or None if no values were collected yet
        """
        return self._shared_data.current

    # TODO what is the difference betwenn this and on_start?
    def on_subscribe(
        self,
        action: Callable[[], None] | Callable[[], Coroutine[Any, Any, None]],
    ) -> Self:
        """Register to receive a callback to be called when the observable starts emitting

        Args:
            action (Callable[[], None] | Callable[[], Coroutine[Any, Any, None]]): Callback

        Returns:
            Self: modified observable
        """
        self._on_subscribe_actions.append(action)
        return self

    def on_start(self, action: SyncAction[T] | AsyncAction[T]) -> Self:
        """Register a callback action to be called when the observable starts emitting

        Args:
            action (SyncAction[T] | AsyncAction[T]): Callback action

        Returns:
            Self: modified observable
        """
        self._on_start_actions.append(action)
        return self

    ####################################################################################################################
    ##### Terminal Operators
    ####################################################################################################################

    def observe(self, replay: int = 1, debug_id: str | None = None) -> Observer[T]:
        """Get an async generator to yield values from the observable

        Args:
            replay (int): how many values to replay from cache. Defaults to 1.
            debug_id (str | None): Identifier for debug logging. Defaults to None (will use generated UUID).

        Returns:
            Observer[T]: async generator to yield values from the observable
        """
        # Create the enhanced generator with a unique ID
        return Observer(self, uuid1(), replay, debug_id=debug_id)

    async def _get_next(self, uuid: UUID) -> T:
        """Get the next per-observer value

        Args:
            uuid (UUID): observer identifier

        Raises:
            RuntimeError: Observer ended without receiving any values

        Returns:
            T: Latest per-observer value
        """
        while True:
            # If this is the first time entering, notify all on-subscribe listeners
            if self._count == 0:
                for action in self._on_subscribe_actions:
                    if iscoroutinefunction(action):
                        await action()
                    action()

            # Acquire the condition and read the per-collector value
            async with self._shared_data:
                if uuid not in self._shared_data.q_dict:
                    logger.error("Attempted to get value from a non-registered observer.")
                    raise RuntimeError("Observer has not been added!")
                q = self._shared_data.q_dict[uuid]
            # Note! This can't be called inside shared data context as it will cause a deadlock. We've already retrieved
            # the q here which is itself coroutine-safe so just await it.
            value = await q.get()
            self._count += 1

            # If this is the first value, notify on start listeners
            if self._count == 1:
                for action in self._on_start_actions:  # type: ignore
                    self._mux_action(action, value)  # type: ignore
            # Notify per-value actions
            for action in self._per_value_actions:  # type: ignore
                action(value)  # type: ignore
            # We've made it! Return the continuing value
            return value



# network_management_pb2.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Apr 24 17:26:18 UTC 2025

"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_sym_db = _symbol_database.Default()
from . import response_generic_pb2 as response__generic__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x18network_management.proto\x12\nopen_gopro\x1a\x16response_generic.proto"R\n\x16NotifProvisioningState\x128\n\x12provisioning_state\x18\x01 \x02(\x0e2\x1c.open_gopro.EnumProvisioning"\x8d\x01\n\x12NotifStartScanning\x120\n\x0escanning_state\x18\x01 \x02(\x0e2\x18.open_gopro.EnumScanning\x12\x0f\n\x07scan_id\x18\x02 \x01(\x05\x12\x15\n\rtotal_entries\x18\x03 \x01(\x05\x12\x1d\n\x15total_configured_ssid\x18\x04 \x02(\x05"\x1e\n\x0eRequestConnect\x12\x0c\n\x04ssid\x18\x01 \x02(\t"\xae\x01\n\x11RequestConnectNew\x12\x0c\n\x04ssid\x18\x01 \x02(\t\x12\x10\n\x08password\x18\x02 \x02(\t\x12\x11\n\tstatic_ip\x18\x03 \x01(\x0c\x12\x0f\n\x07gateway\x18\x04 \x01(\x0c\x12\x0e\n\x06subnet\x18\x05 \x01(\x0c\x12\x13\n\x0bdns_primary\x18\x06 \x01(\x0c\x12\x15\n\rdns_secondary\x18\x07 \x01(\x0c\x12\x19\n\x11bypass_eula_check\x18\n \x01(\x08"P\n\x13RequestGetApEntries\x12\x13\n\x0bstart_index\x18\x01 \x02(\x05\x12\x13\n\x0bmax_entries\x18\x02 \x02(\x05\x12\x0f\n\x07scan_id\x18\x03 \x02(\x05"\x17\n\x15RequestReleaseNetwork"\x12\n\x10RequestStartScan"\x93\x01\n\x0fResponseConnect\x12-\n\x06result\x18\x01 \x02(\x0e2\x1d.open_gopro.EnumResultGeneric\x128\n\x12provisioning_state\x18\x02 \x02(\x0e2\x1c.open_gopro.EnumProvisioning\x12\x17\n\x0ftimeout_seconds\x18\x03 \x02(\x05"\x96\x01\n\x12ResponseConnectNew\x12-\n\x06result\x18\x01 \x02(\x0e2\x1d.open_gopro.EnumResultGeneric\x128\n\x12provisioning_state\x18\x02 \x02(\x0e2\x1c.open_gopro.EnumProvisioning\x12\x17\n\x0ftimeout_seconds\x18\x03 \x02(\x05"\x84\x02\n\x14ResponseGetApEntries\x12-\n\x06result\x18\x01 \x02(\x0e2\x1d.open_gopro.EnumResultGeneric\x12\x0f\n\x07scan_id\x18\x02 \x02(\x05\x12;\n\x07entries\x18\x03 \x03(\x0b2*.open_gopro.ResponseGetApEntries.ScanEntry\x1ao\n\tScanEntry\x12\x0c\n\x04ssid\x18\x01 \x02(\t\x12\x1c\n\x14signal_strength_bars\x18\x02 \x02(\x05\x12\x1c\n\x14signal_frequency_mhz\x18\x04 \x02(\x05\x12\x18\n\x10scan_entry_flags\x18\x05 \x02(\x05"x\n\x15ResponseStartScanning\x12-\n\x06result\x18\x01 \x02(\x0e2\x1d.open_gopro.EnumResultGeneric\x120\n\x0escanning_state\x18\x02 \x02(\x0e2\x18.open_gopro.EnumScanning*\xb5\x03\n\x10EnumProvisioning\x12\x18\n\x14PROVISIONING_UNKNOWN\x10\x00\x12\x1e\n\x1aPROVISIONING_NEVER_STARTED\x10\x01\x12\x18\n\x14PROVISIONING_STARTED\x10\x02\x12"\n\x1ePROVISIONING_ABORTED_BY_SYSTEM\x10\x03\x12"\n\x1ePROVISIONING_CANCELLED_BY_USER\x10\x04\x12\x1f\n\x1bPROVISIONING_SUCCESS_NEW_AP\x10\x05\x12\x1f\n\x1bPROVISIONING_SUCCESS_OLD_AP\x10\x06\x12*\n&PROVISIONING_ERROR_FAILED_TO_ASSOCIATE\x10\x07\x12$\n PROVISIONING_ERROR_PASSWORD_AUTH\x10\x08\x12$\n PROVISIONING_ERROR_EULA_BLOCKING\x10\t\x12"\n\x1ePROVISIONING_ERROR_NO_INTERNET\x10\n\x12\'\n#PROVISIONING_ERROR_UNSUPPORTED_TYPE\x10\x0b*\xac\x01\n\x0cEnumScanning\x12\x14\n\x10SCANNING_UNKNOWN\x10\x00\x12\x1a\n\x16SCANNING_NEVER_STARTED\x10\x01\x12\x14\n\x10SCANNING_STARTED\x10\x02\x12\x1e\n\x1aSCANNING_ABORTED_BY_SYSTEM\x10\x03\x12\x1e\n\x1aSCANNING_CANCELLED_BY_USER\x10\x04\x12\x14\n\x10SCANNING_SUCCESS\x10\x05*\xb2\x01\n\x12EnumScanEntryFlags\x12\x12\n\x0eSCAN_FLAG_OPEN\x10\x00\x12\x1b\n\x17SCAN_FLAG_AUTHENTICATED\x10\x01\x12\x18\n\x14SCAN_FLAG_CONFIGURED\x10\x02\x12\x17\n\x13SCAN_FLAG_BEST_SSID\x10\x04\x12\x18\n\x14SCAN_FLAG_ASSOCIATED\x10\x08\x12\x1e\n\x1aSCAN_FLAG_UNSUPPORTED_TYPE\x10\x10'
)
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "network_management_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _ENUMPROVISIONING._serialized_start = 1317
    _ENUMPROVISIONING._serialized_end = 1754
    _ENUMSCANNING._serialized_start = 1757
    _ENUMSCANNING._serialized_end = 1929
    _ENUMSCANENTRYFLAGS._serialized_start = 1932
    _ENUMSCANENTRYFLAGS._serialized_end = 2110
    _NOTIFPROVISIONINGSTATE._serialized_start = 64
    _NOTIFPROVISIONINGSTATE._serialized_end = 146
    _NOTIFSTARTSCANNING._serialized_start = 149
    _NOTIFSTARTSCANNING._serialized_end = 290
    _REQUESTCONNECT._serialized_start = 292
    _REQUESTCONNECT._serialized_end = 322
    _REQUESTCONNECTNEW._serialized_start = 325
    _REQUESTCONNECTNEW._serialized_end = 499
    _REQUESTGETAPENTRIES._serialized_start = 501
    _REQUESTGETAPENTRIES._serialized_end = 581
    _REQUESTRELEASENETWORK._serialized_start = 583
    _REQUESTRELEASENETWORK._serialized_end = 606
    _REQUESTSTARTSCAN._serialized_start = 608
    _REQUESTSTARTSCAN._serialized_end = 626
    _RESPONSECONNECT._serialized_start = 629
    _RESPONSECONNECT._serialized_end = 776
    _RESPONSECONNECTNEW._serialized_start = 779
    _RESPONSECONNECTNEW._serialized_end = 929
    _RESPONSEGETAPENTRIES._serialized_start = 932
    _RESPONSEGETAPENTRIES._serialized_end = 1192
    _RESPONSEGETAPENTRIES_SCANENTRY._serialized_start = 1081
    _RESPONSEGETAPENTRIES_SCANENTRY._serialized_end = 1192
    _RESPONSESTARTSCANNING._serialized_start = 1194
    _RESPONSESTARTSCANNING._serialized_end = 1314



# live_streaming_pb2.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Apr 24 17:26:18 UTC 2025

"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x14live_streaming.proto\x12\nopen_gopro"\xcb\x04\n\x16NotifyLiveStreamStatus\x12<\n\x12live_stream_status\x18\x01 \x01(\x0e2 .open_gopro.EnumLiveStreamStatus\x12:\n\x11live_stream_error\x18\x02 \x01(\x0e2\x1f.open_gopro.EnumLiveStreamError\x12\x1a\n\x12live_stream_encode\x18\x03 \x01(\x08\x12\x1b\n\x13live_stream_bitrate\x18\x04 \x01(\x05\x12K\n\'live_stream_window_size_supported_array\x18\x05 \x03(\x0e2\x1a.open_gopro.EnumWindowSize\x12$\n\x1clive_stream_encode_supported\x18\x06 \x01(\x08\x12(\n live_stream_max_lens_unsupported\x18\x07 \x01(\x08\x12*\n"live_stream_minimum_stream_bitrate\x18\x08 \x01(\x05\x12*\n"live_stream_maximum_stream_bitrate\x18\t \x01(\x05\x12"\n\x1alive_stream_lens_supported\x18\n \x01(\x08\x12>\n live_stream_lens_supported_array\x18\x0b \x03(\x0e2\x14.open_gopro.EnumLens\x12%\n\x1dlive_stream_protune_supported\x18\r \x01(\x08"\xbc\x01\n\x1aRequestGetLiveStreamStatus\x12M\n\x1bregister_live_stream_status\x18\x01 \x03(\x0e2(.open_gopro.EnumRegisterLiveStreamStatus\x12O\n\x1dunregister_live_stream_status\x18\x02 \x03(\x0e2(.open_gopro.EnumRegisterLiveStreamStatus"\xe6\x01\n\x18RequestSetLiveStreamMode\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0e\n\x06encode\x18\x02 \x01(\x08\x12/\n\x0bwindow_size\x18\x03 \x01(\x0e2\x1a.open_gopro.EnumWindowSize\x12\x0c\n\x04cert\x18\x06 \x01(\x0c\x12\x17\n\x0fminimum_bitrate\x18\x07 \x01(\x05\x12\x17\n\x0fmaximum_bitrate\x18\x08 \x01(\x05\x12\x18\n\x10starting_bitrate\x18\t \x01(\x05\x12"\n\x04lens\x18\n \x01(\x0e2\x14.open_gopro.EnumLens*>\n\x08EnumLens\x12\r\n\tLENS_WIDE\x10\x00\x12\x0f\n\x0bLENS_LINEAR\x10\x04\x12\x12\n\x0eLENS_SUPERVIEW\x10\x03*\xde\x03\n\x13EnumLiveStreamError\x12\x1a\n\x16LIVE_STREAM_ERROR_NONE\x10\x00\x12\x1d\n\x19LIVE_STREAM_ERROR_NETWORK\x10\x01\x12"\n\x1eLIVE_STREAM_ERROR_CREATESTREAM\x10\x02\x12!\n\x1dLIVE_STREAM_ERROR_OUTOFMEMORY\x10\x03\x12!\n\x1dLIVE_STREAM_ERROR_INPUTSTREAM\x10\x04\x12\x1e\n\x1aLIVE_STREAM_ERROR_INTERNET\x10\x05\x12\x1f\n\x1bLIVE_STREAM_ERROR_OSNETWORK\x10\x06\x12,\n(LIVE_STREAM_ERROR_SELECTEDNETWORKTIMEOUT\x10\x07\x12#\n\x1fLIVE_STREAM_ERROR_SSL_HANDSHAKE\x10\x08\x12$\n LIVE_STREAM_ERROR_CAMERA_BLOCKED\x10\t\x12\x1d\n\x19LIVE_STREAM_ERROR_UNKNOWN\x10\n\x12"\n\x1eLIVE_STREAM_ERROR_SD_CARD_FULL\x10(\x12%\n!LIVE_STREAM_ERROR_SD_CARD_REMOVED\x10)*\xa3\x02\n\x14EnumLiveStreamStatus\x12\x1a\n\x16LIVE_STREAM_STATE_IDLE\x10\x00\x12\x1c\n\x18LIVE_STREAM_STATE_CONFIG\x10\x01\x12\x1b\n\x17LIVE_STREAM_STATE_READY\x10\x02\x12\x1f\n\x1bLIVE_STREAM_STATE_STREAMING\x10\x03\x12&\n"LIVE_STREAM_STATE_COMPLETE_STAY_ON\x10\x04\x12$\n LIVE_STREAM_STATE_FAILED_STAY_ON\x10\x05\x12"\n\x1eLIVE_STREAM_STATE_RECONNECTING\x10\x06\x12!\n\x1dLIVE_STREAM_STATE_UNAVAILABLE\x10\x07*\xbc\x01\n\x1cEnumRegisterLiveStreamStatus\x12&\n"REGISTER_LIVE_STREAM_STATUS_STATUS\x10\x01\x12%\n!REGISTER_LIVE_STREAM_STATUS_ERROR\x10\x02\x12$\n REGISTER_LIVE_STREAM_STATUS_MODE\x10\x03\x12\'\n#REGISTER_LIVE_STREAM_STATUS_BITRATE\x10\x04*P\n\x0eEnumWindowSize\x12\x13\n\x0fWINDOW_SIZE_480\x10\x04\x12\x13\n\x0fWINDOW_SIZE_720\x10\x07\x12\x14\n\x10WINDOW_SIZE_1080\x10\x0c'
)
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "live_streaming_pb2", globals())
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _ENUMLENS._serialized_start = 1050
    _ENUMLENS._serialized_end = 1112
    _ENUMLIVESTREAMERROR._serialized_start = 1115
    _ENUMLIVESTREAMERROR._serialized_end = 1593
    _ENUMLIVESTREAMSTATUS._serialized_start = 1596
    _ENUMLIVESTREAMSTATUS._serialized_end = 1887
    _ENUMREGISTERLIVESTREAMSTATUS._serialized_start = 1890
    _ENUMREGISTERLIVESTREAMSTATUS._serialized_end = 2078
    _ENUMWINDOWSIZE._serialized_start = 2080
    _ENUMWINDOWSIZE._serialized_end = 2160
    _NOTIFYLIVESTREAMSTATUS._serialized_start = 37
    _NOTIFYLIVESTREAMSTATUS._serialized_end = 624
    _REQUESTGETLIVESTREAMSTATUS._serialized_start = 627
    _REQUESTGETLIVESTREAMSTATUS._serialized_end = 815
    _REQUESTSETLIVESTREAMMODE._serialized_start = 818
    _REQUESTSETLIVESTREAMMODE._serialized_end = 1048



# responses.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Wed, Sep  1, 2021  5:05:49 PM

"""Any responses that are returned from GoPro commands."""

from __future__ import annotations

import enum
import logging
from dataclasses import dataclass
from typing import Generic, TypeVar

from open_gopro.domain.enum import GoProIntEnum
from open_gopro.domain.types import ResponseType
from open_gopro.models.constants import ErrorCode, QueryCmdId
from open_gopro.util import pretty_print

logger = logging.getLogger(__name__)

T = TypeVar("T")


class GoProBlePacketHeader(enum.Enum):
    """Packet Headers."""

    GENERAL = 0b00
    EXT_13 = 0b01
    EXT_16 = 0b10
    RESERVED = 0b11
    CONT = enum.auto()


@dataclass
class GoProResp(Generic[T]):
    """The object used to encapsulate all GoPro responses.

    It consists of several common properties / attribute and a data attribute that varies per response.

    >>> gopro = WirelessGoPro()
    >>> await gopro.open()
    >>> response = await (gopro.ble_setting.resolution).get_value()
    >>> print(response)

    Now let's inspect the responses various attributes / properties:

    >>> print(response.status)
    ErrorCode.SUCCESS
    >>> print(response.ok)
    True
    >>> print(response.identifier)
    QueryCmdId.GET_SETTING_VAL
    >>> print(response.protocol)
    Protocol.BLE

    Now let's print it's data as (as JSON):

    >>> print(response)
    {
        "id" : "QueryCmdId.GET_SETTING_VAL",
        "status" : "ErrorCode.SUCCESS",
        "protocol" : "Protocol.BLE",
        "data" : {
            "SettingId.RESOLUTION" : "Resolution.RES_4K_16_9",
        },
    }

    Attributes:
        protocol (GoProResp.Protocol): protocol response was received on
        status (ErrorCode): status of response
        data (T): parsed response data
        identifier (ResponseType): response identifier, the type of which will vary depending on the response
    """

    class Protocol(enum.Enum):
        """Protocol that Command will be sent on."""

        BLE = "BLE"
        HTTP = "HTTP"

    protocol: GoProResp.Protocol
    status: ErrorCode
    data: T
    identifier: ResponseType

    def _as_dict(self) -> dict:
        """Represent the response as dictionary, merging it's data and meta information

        Returns:
            dict: dict representation
        """
        d = {
            "id": self.identifier,
            "status": self.status,
            "protocol": self.protocol,
        }
        if self.data:
            d["data"] = self.data  # type: ignore
        return d

    def __eq__(self, obj: object) -> bool:
        if isinstance(obj, GoProIntEnum):
            return self.identifier == obj
        if isinstance(obj, GoProResp):
            return self.identifier == obj.identifier
        raise TypeError("Equal can only compare GoProResp and ResponseType")

    def __str__(self) -> str:
        return pretty_print(self._as_dict())

    def __repr__(self) -> str:
        return f"GoProResp <{str(self.identifier)}>"

    @property
    def ok(self) -> bool:
        """Are there any errors in this response?

        Returns:
            bool: True if the response is ok (i.e. there are no errors), False otherwise
        """
        return self.status in [ErrorCode.SUCCESS, ErrorCode.UNKNOWN]

    @property
    def _is_push(self) -> bool:
        """Was this response an asynchronous push?

        Returns:
            bool: True if yes, False otherwise
        """
        return self.identifier in [
            QueryCmdId.STATUS_VAL_PUSH,
            QueryCmdId.SETTING_VAL_PUSH,
            QueryCmdId.SETTING_CAPABILITY_PUSH,
        ]

    @property
    def _is_query(self) -> bool:
        """Is this response to a settings / status query?

        Returns:
            bool: True if yes, False otherwise
        """
        return isinstance(self.identifier, QueryCmdId)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Defines the structure of protobuf messages for Camera On the Home Network
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
from . import response_generic_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _EnumCOHNStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumCOHNStatusEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumCOHNStatus.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    COHN_UNPROVISIONED: _EnumCOHNStatus.ValueType
    COHN_PROVISIONED: _EnumCOHNStatus.ValueType

class EnumCOHNStatus(_EnumCOHNStatus, metaclass=_EnumCOHNStatusEnumTypeWrapper): ...

COHN_UNPROVISIONED: EnumCOHNStatus.ValueType
COHN_PROVISIONED: EnumCOHNStatus.ValueType
global___EnumCOHNStatus = EnumCOHNStatus

class _EnumCOHNNetworkState:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumCOHNNetworkStateEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumCOHNNetworkState.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    COHN_STATE_Init: _EnumCOHNNetworkState.ValueType
    COHN_STATE_Error: _EnumCOHNNetworkState.ValueType
    COHN_STATE_Exit: _EnumCOHNNetworkState.ValueType
    COHN_STATE_Idle: _EnumCOHNNetworkState.ValueType
    COHN_STATE_NetworkConnected: _EnumCOHNNetworkState.ValueType
    COHN_STATE_NetworkDisconnected: _EnumCOHNNetworkState.ValueType
    COHN_STATE_ConnectingToNetwork: _EnumCOHNNetworkState.ValueType
    COHN_STATE_Invalid: _EnumCOHNNetworkState.ValueType

class EnumCOHNNetworkState(_EnumCOHNNetworkState, metaclass=_EnumCOHNNetworkStateEnumTypeWrapper): ...

COHN_STATE_Init: EnumCOHNNetworkState.ValueType
COHN_STATE_Error: EnumCOHNNetworkState.ValueType
COHN_STATE_Exit: EnumCOHNNetworkState.ValueType
COHN_STATE_Idle: EnumCOHNNetworkState.ValueType
COHN_STATE_NetworkConnected: EnumCOHNNetworkState.ValueType
COHN_STATE_NetworkDisconnected: EnumCOHNNetworkState.ValueType
COHN_STATE_ConnectingToNetwork: EnumCOHNNetworkState.ValueType
COHN_STATE_Invalid: EnumCOHNNetworkState.ValueType
global___EnumCOHNNetworkState = EnumCOHNNetworkState

@typing_extensions.final
class RequestGetCOHNStatus(google.protobuf.message.Message):
    """*
    Get the current COHN status.

    Response: @ref NotifyCOHNStatus

    Additionally, asynchronous updates can also be registered to return more @ref NotifyCOHNStatus when a value
    changes.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REGISTER_COHN_STATUS_FIELD_NUMBER: builtins.int
    register_cohn_status: builtins.bool
    "1 to register, 0 to unregister"

    def __init__(self, *, register_cohn_status: builtins.bool | None = ...) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["register_cohn_status", b"register_cohn_status"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing_extensions.Literal["register_cohn_status", b"register_cohn_status"]
    ) -> None: ...

global___RequestGetCOHNStatus = RequestGetCOHNStatus

@typing_extensions.final
class NotifyCOHNStatus(google.protobuf.message.Message):
    """
    Current COHN status triggered by a @ref RequestGetCOHNStatus
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STATUS_FIELD_NUMBER: builtins.int
    STATE_FIELD_NUMBER: builtins.int
    USERNAME_FIELD_NUMBER: builtins.int
    PASSWORD_FIELD_NUMBER: builtins.int
    IPADDRESS_FIELD_NUMBER: builtins.int
    ENABLED_FIELD_NUMBER: builtins.int
    SSID_FIELD_NUMBER: builtins.int
    MACADDRESS_FIELD_NUMBER: builtins.int
    status: global___EnumCOHNStatus.ValueType
    "Current COHN status"
    state: global___EnumCOHNNetworkState.ValueType
    "Current COHN network state"
    username: builtins.str
    "Username used for http basic auth header"
    password: builtins.str
    "Password used for http basic auth header"
    ipaddress: builtins.str
    "Camera's IP address on the local network"
    enabled: builtins.bool
    "Is COHN currently enabled?"
    ssid: builtins.str
    "Currently connected SSID"
    macaddress: builtins.str
    "MAC address of the wifi adapter"

    def __init__(
        self,
        *,
        status: global___EnumCOHNStatus.ValueType | None = ...,
        state: global___EnumCOHNNetworkState.ValueType | None = ...,
        username: builtins.str | None = ...,
        password: builtins.str | None = ...,
        ipaddress: builtins.str | None = ...,
        enabled: builtins.bool | None = ...,
        ssid: builtins.str | None = ...,
        macaddress: builtins.str | None = ...
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "enabled",
            b"enabled",
            "ipaddress",
            b"ipaddress",
            "macaddress",
            b"macaddress",
            "password",
            b"password",
            "ssid",
            b"ssid",
            "state",
            b"state",
            "status",
            b"status",
            "username",
            b"username",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "enabled",
            b"enabled",
            "ipaddress",
            b"ipaddress",
            "macaddress",
            b"macaddress",
            "password",
            b"password",
            "ssid",
            b"ssid",
            "state",
            b"state",
            "status",
            b"status",
            "username",
            b"username",
        ],
    ) -> None: ...

global___NotifyCOHNStatus = NotifyCOHNStatus

@typing_extensions.final
class RequestCreateCOHNCert(google.protobuf.message.Message):
    """*
    Create the Camera On the Home Network SSL/TLS certificate.

    Returns a @ref ResponseGeneric with the status of the creation
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OVERRIDE_FIELD_NUMBER: builtins.int
    override: builtins.bool
    "Override current provisioning and create new cert"

    def __init__(self, *, override: builtins.bool | None = ...) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["override", b"override"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["override", b"override"]) -> None: ...

global___RequestCreateCOHNCert = RequestCreateCOHNCert

@typing_extensions.final
class RequestClearCOHNCert(google.protobuf.message.Message):
    """*
    Clear the COHN certificate.

    Returns a @ref ResponseGeneric with the status of the clear
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___RequestClearCOHNCert = RequestClearCOHNCert

@typing_extensions.final
class RequestCOHNCert(google.protobuf.message.Message):
    """*
    Get the COHN certificate.

    Returns a @ref ResponseCOHNCert
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___RequestCOHNCert = RequestCOHNCert

@typing_extensions.final
class ResponseCOHNCert(google.protobuf.message.Message):
    """
    COHN Certificate response triggered by @ref RequestCOHNCert
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    RESULT_FIELD_NUMBER: builtins.int
    CERT_FIELD_NUMBER: builtins.int
    result: response_generic_pb2.EnumResultGeneric.ValueType
    "Was request successful?"
    cert: builtins.str
    "Root CA cert (ASCII text)"

    def __init__(
        self, *, result: response_generic_pb2.EnumResultGeneric.ValueType | None = ..., cert: builtins.str | None = ...
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["cert", b"cert", "result", b"result"]
    ) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cert", b"cert", "result", b"result"]) -> None: ...

global___ResponseCOHNCert = ResponseCOHNCert

@typing_extensions.final
class RequestSetCOHNSetting(google.protobuf.message.Message):
    """*
    Configure a COHN Setting

    Returns a @ref ResponseGeneric
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COHN_ACTIVE_FIELD_NUMBER: builtins.int
    cohn_active: builtins.bool
    "*\n    1 to enable COHN, 0 to disable COHN\n\n    When set to 1, STA Mode connection will be dropped and camera will not automatically re-connect for COHN.\n    "

    def __init__(self, *, cohn_active: builtins.bool | None = ...) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["cohn_active", b"cohn_active"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["cohn_active", b"cohn_active"]) -> None: ...

global___RequestSetCOHNSetting = RequestSetCOHNSetting



# livestream.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Wed, Sep  1, 2021  5:05:45 PM

"""Example to start and view a livestream"""

import argparse
import asyncio
from typing import Any

from rich.console import Console

from open_gopro import WirelessGoPro
from open_gopro.models import constants, proto
from open_gopro.util import add_cli_args_and_parse, ainput
from open_gopro.util.logger import setup_logging

console = Console()  # rich consoler printer


async def main(args: argparse.Namespace) -> None:
    setup_logging(__name__, args.log)

    async with WirelessGoPro(args.identifier, enable_wifi=False) as gopro:
        await gopro.ble_command.set_shutter(shutter=constants.Toggle.DISABLE)
        await gopro.ble_command.register_livestream_status(
            register=[proto.EnumRegisterLiveStreamStatus.REGISTER_LIVE_STREAM_STATUS_STATUS]
        )

        console.print(f"[yellow]Connecting to {args.ssid}...")
        await gopro.access_point.connect(args.ssid, args.password)

        # Start livestream
        livestream_is_ready = asyncio.Event()

        async def wait_for_livestream_start(_: Any, update: proto.NotifyLiveStreamStatus) -> None:
            if update.live_stream_status == proto.EnumLiveStreamStatus.LIVE_STREAM_STATE_READY:
                livestream_is_ready.set()

        console.print("[yellow]Configuring livestream...")
        gopro.register_update(wait_for_livestream_start, constants.ActionId.LIVESTREAM_STATUS_NOTIF)
        await gopro.ble_command.set_livestream_mode(
            url=args.url,
            window_size=args.resolution,
            minimum_bitrate=args.min_bit,
            maximum_bitrate=args.max_bit,
            starting_bitrate=args.start_bit,
            encode=args.encode,
            lens=args.fov,
        )

        # Wait to receive livestream started status
        console.print("[yellow]Waiting for livestream to be ready...\n")
        await livestream_is_ready.wait()

        # TODO Is this still needed?
        await asyncio.sleep(2)

        console.print("[yellow]Starting livestream")
        assert (await gopro.ble_command.set_shutter(shutter=constants.Toggle.ENABLE)).ok

        console.print("[yellow]Livestream is now streaming and should be available for viewing.")
        await ainput("Press enter to stop livestreaming...\n")

        await gopro.ble_command.set_shutter(shutter=constants.Toggle.DISABLE)
        await gopro.ble_command.release_network()


def parse_arguments() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        description="Connect to the GoPro via BLE only, configure then start a Livestream, then display it with CV2."
    )
    parser.add_argument("ssid", type=str, help="WiFi SSID to connect to.")
    parser.add_argument("password", type=str, help="Password of WiFi SSID.")
    parser.add_argument("url", type=str, help="RTMP server URL to stream to.")
    parser.add_argument("--min_bit", type=int, help="Minimum bitrate.", default=1000)
    parser.add_argument("--max_bit", type=int, help="Maximum bitrate.", default=1000)
    parser.add_argument("--start_bit", type=int, help="Starting bitrate.", default=1000)
    parser.add_argument(
        "--resolution", help="Resolution.", choices=list(proto.EnumWindowSize.values()), default=None, type=int
    )
    parser.add_argument("--fov", help="Field of View.", choices=list(proto.EnumLens.values()), default=None, type=int)
    parser.add_argument("--encode", help="Save video to sdcard.", action=argparse.BooleanOptionalAction)
    parser.set_defaults(encode=True)
    return add_cli_args_and_parse(parser, wifi=False)


def entrypoint() -> None:
    asyncio.run(main(parse_arguments()))


if __name__ == "__main__":
    entrypoint()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Commands to query and manipulate media files
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
from . import response_generic_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class RequestGetLastCapturedMedia(google.protobuf.message.Message):
    """*
    Get the last captured media filename

    Returns a @ref ResponseLastCapturedMedia
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(self) -> None: ...

global___RequestGetLastCapturedMedia = RequestGetLastCapturedMedia

@typing_extensions.final
class ResponseLastCapturedMedia(google.protobuf.message.Message):
    """*
    The Last Captured Media

    Message is sent in response to a @ref RequestGetLastCapturedMedia.

    This contains the relative path of the last captured media starting from the `DCIM` directory on the SDCard. Depending
    on the type of media captured, it will return:

    - The single media path for single photo/video media
    - The path to the first captured media in the group for grouped media
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    RESULT_FIELD_NUMBER: builtins.int
    MEDIA_FIELD_NUMBER: builtins.int
    result: response_generic_pb2.EnumResultGeneric.ValueType
    "Was the request successful?"

    @property
    def media(self) -> response_generic_pb2.Media:
        """*
        Last captured media if result is RESULT_SUCCESS. Invalid if result is RESULT_RESOURCE_NOT_AVAILBLE.
        """

    def __init__(
        self,
        *,
        result: response_generic_pb2.EnumResultGeneric.ValueType | None = ...,
        media: response_generic_pb2.Media | None = ...
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["media", b"media", "result", b"result"]
    ) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["media", b"media", "result", b"result"]) -> None: ...

global___ResponseLastCapturedMedia = ResponseLastCapturedMedia



# media_list.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Jun 26 18:26:05 UTC 2023

"""Media List and Metadata containers and helper methods"""

from __future__ import annotations

from abc import ABC
from typing import Any

from pydantic import Field, PrivateAttr, field_validator

from open_gopro.domain.types import JsonDict
from open_gopro.models.bases import CustomBaseModel

##############################################################################################################
# Generic
##############################################################################################################


class MediaPath(ABC, CustomBaseModel):
    """Model to represent media path"""

    folder: str  #: directory that media lives in
    file: str  #: media file name (including file extension)

    @property
    def as_path(self) -> str:
        """Return the model as a camera path (folder/file)

        Returns:
            str: camera path
        """
        return f"{self.folder}/{self.file}"

    def __str__(self) -> str:
        return self.as_path


##############################################################################################################
# Metadata
##############################################################################################################


class MediaMetadata(ABC, CustomBaseModel):
    """Base Media Metadata class"""

    content_type: str = Field(alias="ct")  #: Media content type
    creation_timestamp: str = Field(alias="cre")  #: Creation time in seconds since epoch
    file_size: str = Field(alias="s")  #: File size in bytes
    gumi: str = Field(alias="gumi")  #: Globally Unique Media ID
    height: str = Field(alias="h")  #: Height of media in pixels
    width: str = Field(alias="w")  #: Width of media in pixels
    hilight_count: str = Field(alias="hc")  #: Number of hilights in media
    image_stabilization: str = Field(alias="eis")  #: 1 if stabilized, 0 otherwise
    metadata_present: str = Field(alias="mp")  #: 1 if metadata is present, 0 otherwise
    rotate: str = Field(alias="rot")  #: Media rotation
    transcoded: str = Field(alias="tr")  #: 1 if file is transcoded, 0 otherwise
    upload_status: str = Field(alias="us")  #: Whether or not the media file has been uploaded
    media_offload_state: list[str] | None = Field(alias="mos", default=None)  #: List of offload states
    parent_gumi: str | None = Field(alias="pgumi", default=None)  #: Only present if in a clip
    field_of_view: str | None = Field(alias="fov", default=None)  #: Field of View
    lens_config: str | None = Field(alias="lc", default=None)  #: Lens configuration
    lens_projection: str | None = Field(alias="prjn", default=None)  #: Lens projection

    @classmethod
    def from_json(cls, json_str: JsonDict) -> MediaMetadata:
        """Build a metadata object given JSON input

        Args:
            json_str (JsonDict): raw JSON

        Returns:
            MediaMetadata: parsed metadata
        """
        # Choose a field that only exists in video to see if this is a video
        return (VideoMetadata if "ao" in json_str else PhotoMetadata)(**json_str)


class VideoMetadata(MediaMetadata):
    """Metadata for a video file"""

    audio_option: str = Field(alias="ao")  #: Auto, wind, or stereo
    avc_level: str = Field(alias="profile")  #: Advanced Video Codec Level
    avc_profile: str = Field(alias="avc_profile")  #: Advanced Video Code Profile
    clipped: str = Field(alias="cl")  #: 1 if clipped, 0 otherwise
    duration: str = Field(alias="dur")  #: Video duration in seconds
    frame_rate: str = Field(alias="fps")  # Video frame rate in frames / second
    frame_rate_divisor: str = Field(alias="fps_denom")  #: Used to modify frame rate
    hilight_list: list[str] = Field(alias="hi")  #: List of hlights in ms offset from start of video
    lrv_file_size: str = Field(alias="ls")  #: Low Resolution Video file size in bytes. -1 if there is no LRV
    max_auto_hilight_score: str = Field(alias="mahs")  #: Maximum auto-hilight score
    protune_audio: str = Field(alias="pta")  #: 1 if protune audio is present, 0 otherwise
    subsample: str = Field(alias="subsample")  #: 1 if subsampled from other video, 0 otherwise
    progressive: str | None = Field(alias="progr", default=None)  #: 1 if progressive, 0 otherwise


class PhotoMetadata(MediaMetadata):
    """Metadata for a Photo file"""

    raw: str | None = Field(default=None)  #: 1 if photo has raw version, 0 otherwise
    """1 if photo taken with wide dynamic range, 0 otherwise"""
    wide_dynamic_range: str | None = Field(alias="wdr", default=None)
    """1 if photo taken with high dynamic range, 0 otherwise"""
    high_dynamic_range: str | None = Field(alias="hdr", default=None)


##############################################################################################################
# Media List
##############################################################################################################


class MediaItem(CustomBaseModel):
    """Base Media Item class"""

    filename: str = Field(alias="n")  #: Name of media item
    creation_timestamp: str = Field(alias="cre")  #: Creation time in seconds since epoch
    modified_time: str = Field(alias="mod")  #: Time file was last modified in seconds since epoch
    low_res_video_size: str | None = Field(alias="glrv", default=None)  #: Low resolution video size
    lrv_file_size: str | None = Field(alias="ls", default=None)  #: Low resolution file size
    session_id: str | None = Field(alias="id", default=None)  # Media list session identifier
    raw: str | None = Field(default=None)  #: 1 if photo has raw version, 0 (or omitted) otherwise


class GroupedMediaItem(MediaItem):
    """Media Item that is also a grouped item.

    An example of a grouped item is a burst photo.
    """

    group_id: str | None = Field(alias="g", default=None)  #: Group Identifier
    group_size: str | None = Field(alias="s", default=None)  # Number of files in the group
    group_first_member_id: str | None = Field(alias="b", default=None)  # ID of first member in the group
    group_last_member_id: str | None = Field(alias="l", default=None)  #: ID of last member in the group
    group_missing_ids: list[str] | None = Field(alias="m", default=None)  #: File ID's that are missing or deleted
    """(b -> burst, c -> continuous shot, n -> night lapse, t -> time lapse)"""
    group_type: str | None = Field(alias="t", default=None)


class MediaFileSystem(CustomBaseModel):
    """Grouping of media items into filesystem(s)"""

    directory: str = Field(alias="d")  # Directory that the files are in
    file_system: list[MediaItem] = Field(alias="fs")  #: List of files

    @field_validator("file_system", mode="before", check_fields=True)
    @classmethod
    def identify_item(cls, value: list[JsonDict]) -> list[MediaItem]:
        """Validate and convert each item in the file system list

        Args:
            value (list[JsonDict]): List of raw JSON items

        Returns:
            list[MediaItem]: List of parsed MediaItem or GroupedMediaItem objects
        """
        return [(GroupedMediaItem if "g" in item else MediaItem)(**item) for item in value]


class MediaList(CustomBaseModel):
    """Top level media list object"""

    identifier: str = Field(alias="id")  #: String identifier of this media list
    media: list[MediaFileSystem]  #: Media filesystem(s)
    _files: list[MediaItem] = PrivateAttr(default_factory=list)

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        # Modify each file name to use full path
        for directory in self.media:
            for media in directory.file_system:
                media.filename = f"{directory.directory}/{media.filename}"
                self._files.append(media)

    def __contains__(self, key: MediaItem | MediaPath | str) -> bool:
        if isinstance(key, MediaItem):
            return key in self.files
        return str(key) in [m.filename for m in self.files]

    @property
    def files(self) -> list[MediaItem]:
        """Helper method to get list of media items

        Returns:
            list[MediaItem]: all media items in this media list
        """
        return self._files



# base_feature.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Functionality common to all features."""

import asyncio
from abc import ABC, abstractmethod

from open_gopro.api.api import WirelessApi
from open_gopro.gopro_base import GoProBase


class BaseFeature(ABC):
    """Base Feature definition / interface

    Args:
        gopro (GoProBase[WirelessApi]): camera to operate on
        loop (asyncio.AbstractEventLoop): asyncio loop to use for this feature
    """

    def __init__(
        self,
        gopro: GoProBase[WirelessApi],
        loop: asyncio.AbstractEventLoop,
    ) -> None:
        self._gopro = gopro
        self._loop = loop

    @property
    @abstractmethod
    def is_ready(self) -> bool:
        """Is the feature ready to use?

        No other methods (besides wait_for_ready) should be used until the feature is ready

        Returns:
            bool: True if ready, False otherwise
        """

    @abstractmethod
    async def wait_for_ready(self) -> None:
        """Wait until the feature is ready to use"""

    @abstractmethod
    def close(self) -> None:
        """Close the feature for use."""



# __init__.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Tue Sep  7 21:35:53 UTC 2021

"""Open GoPro BLE Interface interface and implementation

isort:skip_file
"""

from open_gopro.domain.exceptions import FailedToFindDevice, ConnectFailed, ConnectionTerminated, ResponseTimeout
from .services import GattDB, Characteristic, Descriptor, Service, BleUUID, UUIDs, CharProps
from .controller import BleDevice, BleHandle, NotiHandlerType, DisconnectHandlerType, BLEController
from .client import BleClient
from .adapters import BleakWrapperController



# gopro_observable.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon May 12 23:03:50 UTC 2025

"""Observable abstraction specifically for use with API components that can be (un)registered for."""

from __future__ import annotations

import logging
from types import TracebackType
from typing import Any, Coroutine, Generic, Self, TypeVar

from open_gopro.domain.communicator_interface import BaseGoProCommunicator
from open_gopro.domain.exceptions import GoProError
from open_gopro.domain.observable import Observable
from open_gopro.domain.types import UpdateType
from open_gopro.models.response import GoProResp

T = TypeVar("T")
I = TypeVar("I")

logger = logging.getLogger(__name__)


class GoproObserverDistinctInitial(Observable[T], Generic[I, T]):
    """Observable for asynchronous notifications where the initial notifications is a different type than proceeding notifications.

    Args:
        gopro (BaseGoProCommunicator): gopro camera to operate on
        update (UpdateType | BaseGoProCommunicator._CompositeRegisterType): the observable's update type
        register_command (Coroutine[Any, Any, GoProResp[I]]): command to call to start receiving values
        unregister_command (Coroutine[Any, Any, Any] | None): Command to call to stop receiving values. Defaults to None.
    """

    def __init__(
        self,
        gopro: BaseGoProCommunicator,
        update: UpdateType | BaseGoProCommunicator._CompositeRegisterType,
        register_command: Coroutine[Any, Any, GoProResp[I]],
        unregister_command: Coroutine[Any, Any, Any] | None = None,
    ) -> None:
        self._gopro = gopro
        self._update = update
        self._register_command = register_command
        self._unregister_command = unregister_command
        self._initial_response: I
        self._is_open = False
        super().__init__(debug_id=str(update))

    @property
    def initial_response(self) -> I:
        """Initial update received from the register command.

        Returns:
            I: Initial update
        """
        return self._initial_response

    async def __aenter__(self) -> GoproObserverDistinctInitial[I, T]:
        if not self._is_open:
            await self.start()
        return self

    async def __aexit__(self, exc_type: BaseException, exc_val: Any, exc_tb: TracebackType) -> None:
        if self._is_open:
            await self.stop()

    async def _emit_value(self, _: UpdateType | BaseGoProCommunicator._CompositeRegisterType, value: T) -> None:
        """Emit a newly received value for observing

        Args:
            value (T): Value of received element.
        """
        await self.emit(value)

    async def start(self: Self) -> Self:
        """Configure the camera to start receiving notifications.

        Raises:
            GoProError: Register command returned a failure update

        Returns:
            Self: modified observable
        """
        self._gopro._register_update(self._emit_value, self._update)
        initial_response = await self._register_command
        if not initial_response.ok:
            raise GoProError(f"Failed to start receiving update ==> {self._update}")
        self._initial_response = initial_response.data
        self._is_open = True
        return self

    async def stop(self) -> None:
        """Configure the camera to stop sending notifications"""
        self._gopro._unregister_update(self._emit_value, self._update)
        if self._unregister_command:
            await self._unregister_command
        self._is_open = False


class GoProObservable(GoproObserverDistinctInitial[T, T]):
    """Observable for asynchronous camera API notifications where all received notifications are the same type.

    Args:
        gopro (BaseGoProCommunicator): gopro camera to operate on
        update (UpdateType | BaseGoProCommunicator._CompositeRegisterType): the update's update type
        register_command (Coroutine[Any, Any, GoProResp[T]]): command to call to start receiving notifications
        unregister_command (Coroutine[Any, Any, Any] | None): Command to call to stop receiving notifications. Defaults to None.
    """

    def __init__(
        self,
        gopro: BaseGoProCommunicator,
        update: UpdateType | BaseGoProCommunicator._CompositeRegisterType,
        register_command: Coroutine[Any, Any, GoProResp[T]],
        unregister_command: Coroutine[Any, Any, Any] | None = None,
    ) -> None:
        super().__init__(gopro, update, register_command, unregister_command)

    async def start(self: Self) -> Self:
        """Configure the camera to start receiving notifications.

        The initial response will be treated and emitted the same as subsequent responses.

        Raises:
            GoProError: Register command returned a failure updates

        Returns:
            Self: modified observable
        """
        self._gopro._register_update(self._emit_value, self._update)
        initial_response = await self._register_command
        if not initial_response.ok:
            raise GoProError(f"Failed to start receiving updates ==> {self._update}")
        self._initial_response = initial_response.data
        await self._emit_value(self._update, initial_response.data)
        self._is_open = True
        return self


class GoProCompositeObservable(GoProObservable[dict[UpdateType | BaseGoProCommunicator._CompositeRegisterType, T]]):
    """Observable for asynchronous camera notifications where all received notifications are the same type.

    Args:
        gopro (BaseGoProCommunicator): gopro camera to operate on
        update (UpdateType | BaseGoProCommunicator._CompositeRegisterType): the updates's update type
        register_command (Coroutine[Any, Any, GoProResp[T]]): command to call to start receiving notifications
        unregister_command (Coroutine[Any, Any, Any] | None): Command to call to stop receiving notifications. Defaults to None.
    """

    async def _emit_value(
        self,
        update: UpdateType | BaseGoProCommunicator._CompositeRegisterType,
        value: T | dict[UpdateType | BaseGoProCommunicator._CompositeRegisterType, T],
    ) -> None:
        """Emit a newly received value for observation

        Args:
            update (UpdateType | BaseGoProCommunicator._CompositeRegisterType): The update type.
            value (T | dict[UpdateType | BaseGoProCommunicator._CompositeRegisterType, T]): The value.
        """
        # This is the initial response which comes as a dict of all values.
        if isinstance(update, BaseGoProCommunicator._CompositeRegisterType) and isinstance(value, dict):
            await self.emit(value)
        # Subsequent responses are just the individual value of the update.
        else:
            await self.emit({update: value})  # type: ignore



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*
Defines the structure of protobuf messages for obtaining preset status
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions
DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _EnumRegisterPresetStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _EnumRegisterPresetStatusEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EnumRegisterPresetStatus.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    REGISTER_PRESET_STATUS_PRESET: _EnumRegisterPresetStatus.ValueType
    "Send notification when properties of a preset change"
    REGISTER_PRESET_STATUS_PRESET_GROUP_ARRAY: _EnumRegisterPresetStatus.ValueType
    "Send notification when properties of a preset group change"

class EnumRegisterPresetStatus(_EnumRegisterPresetStatus, metaclass=_EnumRegisterPresetStatusEnumTypeWrapper): ...

REGISTER_PRESET_STATUS_PRESET: EnumRegisterPresetStatus.ValueType
"Send notification when properties of a preset change"
REGISTER_PRESET_STATUS_PRESET_GROUP_ARRAY: EnumRegisterPresetStatus.ValueType
"Send notification when properties of a preset group change"
global___EnumRegisterPresetStatus = EnumRegisterPresetStatus

@typing_extensions.final
class RequestGetPresetStatus(google.protobuf.message.Message):
    """*
    Get the set of currently available presets and optionally register to be notified when it changes.

    Response: @ref NotifyPresetStatus sent immediately

    Notification: @ref NotifyPresetStatus sent periodically as preset status changes, if registered.

    The preset status changes when:

    - A client changes one of a preset's captioned settings via the API
    - The user exits from a preset's settings UI on the camera (e.g. long-press the preset pill and then press the back arrow)
    - The user creates/deletes/reorders a preset within a group
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REGISTER_PRESET_STATUS_FIELD_NUMBER: builtins.int
    UNREGISTER_PRESET_STATUS_FIELD_NUMBER: builtins.int
    USE_CONSTANT_SETTING_IDS_FIELD_NUMBER: builtins.int

    @property
    def register_preset_status(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___EnumRegisterPresetStatus.ValueType]:
        """Array of Preset statuses to be notified about"""

    @property
    def unregister_preset_status(
        self,
    ) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[global___EnumRegisterPresetStatus.ValueType]:
        """Array of Preset statuses to stop being notified about"""
    use_constant_setting_ids: builtins.bool
    "*\n    Configures the set of Setting IDs that are returned by @ref NotifyPresetStatus.\n\n    Defaults to False if not set\n\n    When False, Setting IDs are tied to their camera UI string representation and may vary between cameras.\n\n    When True, Setting IDs are tied to their functional representation and are consistent across cameras.\n    "

    def __init__(
        self,
        *,
        register_preset_status: collections.abc.Iterable[global___EnumRegisterPresetStatus.ValueType] | None = ...,
        unregister_preset_status: collections.abc.Iterable[global___EnumRegisterPresetStatus.ValueType] | None = ...,
        use_constant_setting_ids: builtins.bool | None = ...
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["use_constant_setting_ids", b"use_constant_setting_ids"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "register_preset_status",
            b"register_preset_status",
            "unregister_preset_status",
            b"unregister_preset_status",
            "use_constant_setting_ids",
            b"use_constant_setting_ids",
        ],
    ) -> None: ...

global___RequestGetPresetStatus = RequestGetPresetStatus



# general.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Mon Jul 31 17:04:07 UTC 2023

"""Other models that don't deserve their own file"""

from __future__ import annotations

import datetime
import json
import tempfile
from base64 import b64encode
from dataclasses import asdict, dataclass
from functools import cached_property
from pathlib import Path

from pydantic import BaseModel, ConfigDict, Field

from open_gopro.models.bases import CustomBaseModel
from open_gopro.models.constants import SettingId, WebcamError, WebcamStatus


class CameraInfo(CustomBaseModel):
    """General camera info"""

    model_config = ConfigDict(protected_namespaces=())
    model_number: int  #: Camera model number
    model_name: str  #: Camera model name as string
    firmware_version: str  #: Complete firmware version
    serial_number: str  #: Camera serial number
    ap_mac_addr: str  #: Camera access point MAC address
    ap_ssid: str  #: Camera access point SSID name


class TzDstDateTime(CustomBaseModel):
    """DST aware datetime"""

    datetime: datetime.datetime
    tzone: int
    dst: bool


class SupportedOption(CustomBaseModel):
    """A supported option in an invalid setting response"""

    display_name: str
    id: int


class WebcamResponse(CustomBaseModel):
    """Common Response from Webcam Commands"""

    status: WebcamStatus | None = Field(default=None)
    error: WebcamError
    setting_id: str | None = Field(default=None)
    supported_options: list[SupportedOption] | None = Field(default=None)


class HttpInvalidSettingResponse(CustomBaseModel):
    """Invalid settings response with optional supported options"""

    error: int
    setting_id: SettingId
    option_id: int | None = Field(default=None)
    supported_options: list[SupportedOption] | None = Field(default=None)


class CohnInfo(BaseModel):
    """Data model to store Camera on the Home Network connection info"""

    ip_address: str
    username: str
    password: str
    certificate: str

    @cached_property
    def auth_token(self) -> str:
        """Get the auth token from the username and password

        Returns:
            str: _description_
        """
        token = b64encode(f"{self.username}:{self.password}".encode("utf-8")).decode("ascii")
        return f"Basic {token}"

    @cached_property
    def certificate_as_path(self) -> Path:
        """Write the certificate to a tempfile and return the tempfile

        This is needed because requests will not take an in-memory certificate string

        Returns:
            Path: Path of temp certificate file
        """
        with tempfile.NamedTemporaryFile(delete=False) as cert_file:
            cert_file.write(self.certificate.encode("utf-8"))
        return Path(cert_file.name)

    def __add__(self, other: CohnInfo) -> CohnInfo:
        return CohnInfo(
            ip_address=other.ip_address or self.ip_address,
            username=other.username or self.username,
            password=other.password or self.password,
            certificate=other.certificate or self.certificate,
        )

    @property
    def is_complete(self) -> bool:
        """Are all of the fields non-empty?

        Returns:
            bool: True if complete, False otherwise
        """
        return bool(self.ip_address) and bool(self.username) and bool(self.password) and bool(self.certificate)


@dataclass(frozen=True)
class ScheduledCapture:
    """Scheduled capture request / status"""

    hour: int
    minute: int
    is_enabled: bool
    is_24_hour: bool

    @classmethod
    def from_datetime(cls, dt: datetime.datetime, is_enabled: bool) -> ScheduledCapture:
        """Helper method to build a ScheduledCapture object from Python standard datetime.datetime

        Args:
            dt (datetime.datetime): datetime to build from
            is_enabled (bool): is / should scheduled capture be enabled on the camera?

        Returns:
            ScheduledCapture: _description_
        """
        return cls(
            hour=dt.hour,
            minute=dt.minute,
            is_enabled=is_enabled,
            is_24_hour=True,
        )

    def __str__(self) -> str:
        return json.dumps(asdict(self), indent=8)

    @classmethod
    def off(cls) -> ScheduledCapture:
        """Helper method to return a pre-filled ScheduledCapture object that can be used to turn scheduled capture off

        Returns:
            ScheduledCapture: object with "is_enabled" set to False
        """
        return cls(0, 0, False, False)



# gopro.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Wed, Sep  1, 2021  5:05:47 PM

"""Implements top level interface to GoPro module."""

from __future__ import annotations

import asyncio
import logging
from typing import Any, Callable, Final

import requests

import open_gopro.network.wifi.mdns_scanner  # Imported this way for pytest monkeypatching
from open_gopro.api import (
    BleCommands,
    BleSettings,
    BleStatuses,
    HttpCommands,
    HttpSettings,
    WiredApi,
)
from open_gopro.domain.communicator_interface import (
    BaseGoProCommunicator,
    GoProWiredInterface,
    Message,
    MessageRules,
)
from open_gopro.domain.exceptions import (
    FailedToFindDevice,
    GoProNotOpened,
    InvalidOpenGoProVersion,
)
from open_gopro.domain.types import CameraState, UpdateCb, UpdateType
from open_gopro.gopro_base import GoProBase
from open_gopro.models import GoProResp, constants
from open_gopro.models.constants import StatusId

logger = logging.getLogger(__name__)

GET_TIMEOUT: Final = 5
HTTP_GET_RETRIES: Final = 5


class WiredGoPro(GoProBase[WiredApi], GoProWiredInterface):
    """The top-level USB interface to a Wired GoPro device.

    See the `Open GoPro SDK <https://gopro.github.io/OpenGoPro/python_sdk>`_ for complete documentation.

    If a serial number is not passed when instantiating, the mDNS server will be queried to find a connected
    GoPro.

    This class also handles:
        - ensuring camera is ready / not encoding before transferring data

    It can be used via context manager:

    >>> async with WiredGoPro() as gopro:
    >>>     # Send some messages now

    Or without:

    >>> gopro = WiredGoPro()
    >>> await gopro.open()
    >>> # Send some messages now

    Args:
        serial (str | None): (at least) last 3 digits of GoPro Serial number. If not set, first GoPro
            discovered from mDNS will be used. Defaults to None
        **kwargs (Any): additional keyword arguments to pass to base class
    """

    _BASE_IP: Final[str] = "172.2{}.1{}{}.51"
    _BASE_ENDPOINT: Final[str] = "http://{ip}:8080/"
    _MDNS_SERVICE_NAME: Final[str] = "_gopro-web._tcp.local."

    def __init__(self, serial: str | None = None, **kwargs: Any) -> None:
        GoProBase.__init__(self, **kwargs)
        GoProWiredInterface.__init__(self)
        self._serial = serial
        # We currently only support version 2.0
        self._wired_api = WiredApi(self)
        self._open = False
        self._poll_period = kwargs.get("poll_period", 2)
        self._encoding = False
        self._busy = False

    async def open(self, timeout: int = 10, retries: int = 1) -> None:
        """Connect to the Wired GoPro Client and prepare it for communication

        Args:
            timeout (int): time (in seconds) before considering connection a failure. Defaults to 10.
            retries (int): number of connection retries. Defaults to 1.

        Raises:
            InvalidOpenGoProVersion: the GoPro camera does not support the correct Open GoPro API version
            FailedToFindDevice: could not auto-discover GoPro via mDNS
        """
        if not self._serial:
            for retry in range(1, retries + 1):
                try:
                    response = await open_gopro.network.wifi.mdns_scanner.find_first_ip_addr(
                        WiredGoPro._MDNS_SERVICE_NAME, timeout
                    )
                    self._serial = response.name.split(".")[0]
                    break
                except FailedToFindDevice as e:
                    if retry == retries:
                        raise e
                    logger.warning(f"Failed to discover GoPro. Retrying #{retry}")

        await self.http_command.wired_usb_control(control=constants.Toggle.ENABLE)
        await self.http_command.set_third_party_client_info()
        # Find and configure API version
        if (version := (await self.http_command.get_open_gopro_api_version()).data) != self.version:
            raise InvalidOpenGoProVersion(version)
        logger.info(f"Using Open GoPro API version {version}")

        # Wait for initial ready state
        await self._wait_for_state({StatusId.ENCODING: False, StatusId.BUSY: False})

        self._open = True

    async def close(self) -> None:
        """Gracefully close the GoPro Client connection"""

    @property
    async def is_ready(self) -> bool:
        """Is gopro ready to receive commands

        Returns:
            bool: yes if ready, no otherwise
        """
        current_state = (await self.http_command.get_camera_state()).data
        self._encoding = bool(current_state[StatusId.ENCODING])
        self._busy = bool(current_state[StatusId.BUSY])
        return not (self._encoding or self._busy)

    @property
    def identifier(self) -> str:
        """Unique identifier for the connected GoPro Client

        Raises:
            GoProNotOpened: serial was not passed to instantiation and IP has not yet been discovered

        Returns:
            str: identifier
        """
        if self._serial:
            return self._serial
        raise GoProNotOpened("IP address has not yet been discovered")

    @property
    def version(self) -> str:
        """The Open GoPro API version of the GoPro Client

        Only Version 2.0 is currently supported.

        Returns:
            str: string version
        """
        return self._api.version

    @property
    def http_command(self) -> HttpCommands:
        """Used to access the USB commands

        Returns:
            HttpCommands: the commands
        """
        return self._api.http_command

    @property
    def http_setting(self) -> HttpSettings:
        """Used to access the USB settings

        Returns:
            HttpSettings: the settings
        """
        return self._api.http_setting

    @property
    def ble_command(self) -> BleCommands:
        """Used to call the BLE commands

        Raises:
            NotImplementedError: Not valid for WiredGoPro
        """
        raise NotImplementedError

    @property
    def ble_setting(self) -> BleSettings:
        """Used to access the BLE settings

        Raises:
            NotImplementedError: Not valid for WiredGoPro
        """
        raise NotImplementedError

    @property
    def ble_status(self) -> BleStatuses:
        """Used to access the BLE statuses

        Raises:
            NotImplementedError: Not valid for WiredGoPro
        """
        raise NotImplementedError

    @property
    def is_open(self) -> bool:
        """Is this client ready for communication?

        Returns:
            bool: True if yes, False if no
        """
        return self._open

    @property
    def is_ble_connected(self) -> bool:
        """Are we connected via BLE to the GoPro device?

        Returns:
            bool: True if yes, False if no
        """
        return False

    @property
    def is_http_connected(self) -> bool:
        """Are we connected via Wifi to the GoPro device?

        Returns:
            bool: True if yes, False if no
        """
        return self.is_open

    def _register_update(
        self, callback: UpdateCb, update: BaseGoProCommunicator._CompositeRegisterType | UpdateType
    ) -> None:
        raise NotImplementedError

    def _unregister_update(
        self, callback: UpdateCb, update: BaseGoProCommunicator._CompositeRegisterType | UpdateType | None = None
    ) -> None:
        raise NotImplementedError

    def register_update(self, callback: UpdateCb, update: UpdateType) -> None:
        """Register for callbacks when an update occurs

        Args:
            callback (UpdateCb): callback to be notified in
            update (UpdateType): update to register for

        Raises:
            NotImplementedError: not yet possible
        """
        raise NotImplementedError

    def unregister_update(self, callback: UpdateCb, update: UpdateType | None = None) -> None:
        """Unregister for asynchronous update(s)

        Args:
            callback (UpdateCb): callback to stop receiving update(s) on
            update (UpdateType | None): updates to unsubscribe for. Defaults to None (all
                updates that use this callback will be unsubscribed).

        Raises:
            NotImplementedError: not yet possible
        """
        raise NotImplementedError

    async def configure_cohn(self, timeout: int = 60) -> bool:
        """Prepare Camera on the Home Network

        Provision if not provisioned
        Then wait for COHN to be connected and ready

        Args:
            timeout (int): time in seconds to wait for COHN to be ready. Defaults to 60.

        Returns:
            bool: True if success, False otherwise

        Raises:
            NotImplementedError: not yet possible
        """
        raise NotImplementedError

    @property
    async def is_cohn_provisioned(self) -> bool:
        """Is COHN currently provisioned?

        Get the current COHN status from the camera

        Returns:
            bool: True if COHN is provisioned, False otherwise

        Raises:
            NotImplementedError: not yet possible
        """
        raise NotImplementedError

    ##########################################################################################################
    #                                 End Public API
    ##########################################################################################################

    async def _enforce_message_rules(
        self, wrapped: Callable, message: Message, rules: MessageRules = MessageRules(), **kwargs: Any
    ) -> GoProResp:
        # Acquire ready lock unless we are initializing or this is a Set Shutter Off command
        if self._should_maintain_state and self.is_open and not rules.is_fastpass(**kwargs):
            # Wait for not encoding and not busy
            logger.trace("Waiting for camera to be ready to receive messages.")  # type: ignore
            await self._wait_for_state({StatusId.ENCODING: False, StatusId.BUSY: False})
            logger.trace("Camera is ready to receive messages")  # type: ignore
            response = await wrapped(message, **kwargs)
        else:  # Either we're not maintaining state, we're not opened yet, or this is a fastpass message
            response = await wrapped(message, **kwargs)

        # Release the lock if we acquired it
        if self._should_maintain_state:
            if response.ok:
                # Is there any special handling required after receiving the response?
                if rules.should_wait_for_encoding_start(**kwargs):
                    logger.trace("Waiting to receive encoding started.")  # type: ignore
                    # Wait for encoding to start
                    await self._wait_for_state({StatusId.ENCODING: True})
        return response

    async def _wait_for_state(self, check: CameraState) -> None:
        """Poll the current state until a variable amount of states are all equal to desired values

        Args:
            check (CameraState): dict{setting / status: value} of settings / statuses and values to wait for
        """
        while True:
            state = (await self.http_command.get_camera_state()).data
            for key, value in check.items():
                if state.get(key) != value:
                    logger.trace(f"Not ready ==> {key} != {value}")  # type: ignore
                    await asyncio.sleep(self._poll_period)
                    break  # Get new state and try again
            else:
                return  # Everything matches. Exit

    @property
    def _api(self) -> WiredApi:
        return self._wired_api

    @property
    def _base_url(self) -> str:
        """Build the base endpoint for USB commands

        Raises:
            GoProNotOpened: The GoPro serial has not yet been set / discovered

        Returns:
            str: base endpoint with URL from serial number
        """
        if not self._serial:
            raise GoProNotOpened("Serial / IP has not yet been discovered")
        return WiredGoPro._BASE_ENDPOINT.format(ip=WiredGoPro._BASE_IP.format(*self._serial[-3:]))

    @property
    def _requests_session(self) -> requests.Session:
        return requests.Session()



# __init__.py/Open GoPro, Version 2.0 (C) Copyright 2021 GoPro, Inc. (http://gopro.com/OpenGoPro).
# This copyright was auto-generated on Thu Feb 20 23:24:52 UTC 2025

"""Constants module"""

from .constants import *
from .settings import SettingId
from .statuses import StatusId
from .uuids import GoProUUID
